//============================================================================
// File          : D3DUtilAppBasic.hpp
// 
// Content       : D3DUtilApp derived class with navigational capabilities
//                      Default navigation
//                      Default path recording/playback
// 
//============================================================================
// 
//============================================================================
#ifndef D3DUtilAppBasic_included_
#define D3DUtilAppBasic_included_
#pragma once


//----------------------------------------------------------------------------
//-- Includes ----------------------------------------------------------------
//----------------------------------------------------------------------------

#include "D3DUtilApp.hpp"
#include "UtilCamera.hpp"
#include "UtilMouseNav.hpp"
#include "UtilCamRecord.hpp"

//----------------------------------------------------------------------------
//-- Forward Declarations ----------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Defines, Constants, Enumerated Types ------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Function Declarations ---------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Typedefs, Structs, Classes ----------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Class D3DUtilAppBasic
//----------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
class D3DUtilAppBasic : public D3DUtilApp
{
public:
	//----------------------------------------------------------------------------
	// protected ctor/dtor
	//----------------------------------------------------------------------------
	D3DUtilAppBasic();
	virtual ~D3DUtilAppBasic();

	//----------------------------------------------------------------------------
	// WndProc. The class must receive all messages from the message loop.
	// completely handled messages are indicated by a return value of S_WNDPROC_HANDLED
	//----------------------------------------------------------------------------
	virtual LRESULT WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& handled);

	//----------------------------------------------------------------------------
	// Scene control
	//----------------------------------------------------------------------------
	virtual HRESULT UpdateScene();

	//----------------------------------------------------------------------------
	// Camera recording - call this to play/record camera movements
	//----------------------------------------------------------------------------
	bool Record(const char *Filename);
	bool Play(const char *Filename);
	bool StopPlay(unsigned *numFrames=NULL, unsigned* timeMS=NULL);
	bool StopRecord();

    //----------------------------------------------------------------------------
	// Access to the navigation controls and the camera
	//----------------------------------------------------------------------------
    UtilMouseNav&        GetNav();
    UtilCamera&          GetCam();

    //----------------------------------------------------------------------------
	// Help Display. When the user hits 'h' they see this string
	//----------------------------------------------------------------------------
	void SetHelpString(const char *helpStr);
	void AppendHelpString(const char *helpStr);

	//----------------------------------------------------------------------------
	// Stat display - call this in RenderScene to show counters, etc.
	//----------------------------------------------------------------------------
	virtual HRESULT ShowStats();
	
	//----------------------------------------------------------------------------
	// Listener response routines.
	//----------------------------------------------------------------------------
    virtual HRESULT Reset();

private:
    UtilCamera          _Cam;
    UtilMouseNav        _Nav;
    UtilCamRecord       _Recorder;
    UtilCamPlay         _Player;
    bool                _ShowHelp;
    std::string         _HelpStr;
};

//----------------------------------------------------------------------------
//-- External Variables ------------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Function Definitions ----------------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// GetNav()
// ---------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
inline
UtilMouseNav& D3DUtilAppBasic::GetNav()
{
    return _Nav;
}

//----------------------------------------------------------------------------
// GetCam()
// ---------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
inline
UtilCamera& D3DUtilAppBasic::GetCam()
{
    return _Cam;
}

#endif
