//-----------------------------------------------------------------------------
// D3DUtilContext.cpp
//-----------------------------------------------------------------------------
// Encapsulates the devices that encompass a D3D8 scene.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// #includes
//-----------------------------------------------------------------------------
#include "stdafx.h"
#include "D3DUtilContext.hpp"

#pragma warning( push )
#pragma warning( disable : 4786)

D3DUtilContext* D3DUtilContext::s_Context = NULL;

//----------------------------------------------------------------------------
// Contructors/Destructors
//----------------------------------------------------------------------------
D3DUtilContext::D3DUtilContext(D3DUtilContextListener* listener) :
Listener(listener)
{
	pD3D = NULL;
	pD3DDevice = NULL;

    Init();

    pD3D = Direct3DCreate8(D3D_SDK_VERSION);
    ASSERT_MSG(pD3D != NULL, "D3DUtilContext: Direct3DCreate8 Failed");

	// Store the windowed format
	D3DDISPLAYMODE mode;
	FAIL_ASSERT(pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &mode));
	WindowedFormat = mode.Format;
}

//----------------------------------------------------------------------------
// Contructors/Destructors
//----------------------------------------------------------------------------
D3DUtilContext::~D3DUtilContext()
{
    // Force a clear if it hasn't already been done. Since we can't return
    // failure on clear, just assert.
    FAIL_ASSERT(Clear());
    SAFE_RELEASE(pD3D);
}

//----------------------------------------------------------------------------
// Init
//----------------------------------------------------------------------------
// Initialize the internal data members
//----------------------------------------------------------------------------
void D3DUtilContext::Init()
{
	ZeroMemory(&D3DPP, sizeof(D3DPP));
	ZeroMemory(&D3DCP, sizeof(D3DCP));
	ZeroMemory(&D3DCaps, sizeof(D3DCaps));
	Stencil = false;

	Active      = false;
	SetRect(&OldWinRect, 0,0,0,0);
	SetRect(&OldClientRect, 0,0,0,0);

	RenderWidth = 0;
	RenderHeight = 0;
}

//----------------------------------------------------------------------------
// InitRenderStates
//----------------------------------------------------------------------------
// Initializes the renderstates arrays to the values that D3D says they are.
// That way we make sure we know what state we start in
//----------------------------------------------------------------------------
HRESULT D3DUtilContext::InitRenderStates()
{
    FAIL_RETURN(pD3DDevice->GetDeviceCaps(&D3DCaps));
    
	float zero = 0.f;
	float one = 1.f;

	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ZENABLE,D3DZB_TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FILLMODE,D3DFILL_SOLID));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_SHADEMODE,D3DSHADE_GOURAUD));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_LINEPATTERN,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ZWRITEENABLE,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_LASTPIXEL,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ZERO));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CCW));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ZFUNC,D3DCMP_LESSEQUAL));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ALPHAREF,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ALPHAFUNC,D3DCMP_ALWAYS));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_DITHERENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_SPECULARENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ZVISIBLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGCOLOR,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGTABLEMODE,D3DFOG_NONE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGSTART,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGEND,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGDENSITY,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_EDGEANTIALIAS,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_ZBIAS,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_RANGEFOGENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILFAIL,D3DSTENCILOP_KEEP));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILZFAIL,D3DSTENCILOP_KEEP));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILPASS,D3DSTENCILOP_KEEP));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILFUNC,D3DCMP_ALWAYS));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILREF,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_STENCILMASK,0xFFFFFFFF));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_TEXTUREFACTOR,0xFF000000));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP0,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP1,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP2,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP3,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP4,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP5,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP6,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_WRAP7,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_CLIPPING,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_LIGHTING,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_AMBIENT,0x00000000));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_FOGVERTEXMODE,D3DFOG_NONE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_COLORVERTEX,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_LOCALVIEWER,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_NORMALIZENORMALS,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE,D3DMCS_COLOR1));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_SPECULARMATERIALSOURCE,D3DMCS_COLOR2));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE,D3DMCS_COLOR2));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE,D3DMCS_MATERIAL));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_VERTEXBLEND,D3DVBF_DISABLE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_CLIPPLANEENABLE,0x0));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_SOFTWAREVERTEXPROCESSING,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSIZE,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSIZE_MIN,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSCALEENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSCALE_A,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSCALE_B,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSCALE_C,*((DWORD*)&zero)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS,TRUE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_MULTISAMPLEMASK,0xFFFFFFFF));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_PATCHEDGESTYLE,D3DPATCHEDGE_DISCRETE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_PATCHSEGMENTS,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_DEBUGMONITORTOKEN,D3DDMT_DISABLE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_POINTSIZE_MAX,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE,FALSE));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE,D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_TWEENFACTOR,*((DWORD*)&one)));
	FAIL_TEST(pD3DDevice->SetRenderState(D3DRS_BLENDOP,D3DBLENDOP_ADD));

	for (int i=0;i<D3DDP_MAXTEXCOORD;++i)
	{
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_COLOROP,i == 0 ? D3DTOP_MODULATE : D3DTOP_DISABLE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_COLORARG1,D3DTA_TEXTURE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_COLORARG2,D3DTA_CURRENT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ALPHAOP,i == 0 ? D3DTOP_SELECTARG1 : D3DTOP_DISABLE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ALPHAARG1,D3DTA_DIFFUSE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ALPHAARG2,D3DTA_CURRENT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVMAT00,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVMAT01,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVMAT10,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVMAT11,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_TEXCOORDINDEX,i | D3DTSS_TCI_PASSTHRU));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ADDRESSU,D3DTADDRESS_WRAP));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ADDRESSV,D3DTADDRESS_WRAP));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BORDERCOLOR,0x00000000));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MAGFILTER,D3DTEXF_POINT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MINFILTER,D3DTEXF_POINT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MIPFILTER,D3DTEXF_NONE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MIPMAPLODBIAS,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MAXMIPLEVEL,0));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_MAXANISOTROPY,1));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVLSCALE,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_BUMPENVLOFFSET,*((DWORD*)&zero)));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_TEXTURETRANSFORMFLAGS,D3DTTFF_DISABLE));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ADDRESSW,D3DTADDRESS_WRAP));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_COLORARG0,D3DTA_CURRENT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_ALPHAARG0,D3DTA_CURRENT));
		FAIL_TEST(pD3DDevice->SetTextureStageState(i,D3DTSS_RESULTARG,D3DTA_CURRENT));
	}

    return D3D_OK;
}

//----------------------------------------------------------------------------
// SetCreationParameters
//----------------------------------------------------------------------------
// Used internally only
//----------------------------------------------------------------------------
void D3DUtilContext::SetCreationParameters(const D3DDEVICE_CREATION_PARAMETERS& d3dcp)
{
	D3DCP = d3dcp;
}

//----------------------------------------------------------------------------
// SetCreationParameters
//----------------------------------------------------------------------------
// Used internally only
//----------------------------------------------------------------------------
void D3DUtilContext::SetPresentParameters(const D3DPRESENT_PARAMETERS& d3dpp)
{
	D3DPP = d3dpp;
	if (d3dpp.EnableAutoDepthStencil &&
		(d3dpp.AutoDepthStencilFormat == D3DFMT_D24S8 ||
		 d3dpp.AutoDepthStencilFormat == D3DFMT_D15S1 ||
		 d3dpp.AutoDepthStencilFormat == D3DFMT_D24X4S4))
	{
		Stencil = true;
	} else
	{
		Stencil = false;
	}

	if (d3dpp.BackBufferWidth && d3dpp.BackBufferHeight)
	{
		RenderWidth = d3dpp.BackBufferWidth;
		RenderHeight = d3dpp.BackBufferHeight;
	} else
	{
		RECT r;
		GetClientRect(d3dpp.hDeviceWindow ? d3dpp.hDeviceWindow : D3DCP.hFocusWindow, &r);
		RenderWidth = r.right-r.left;
		RenderHeight = r.bottom-r.top;
	}
}

//----------------------------------------------------------------------------
// InitDevice
//----------------------------------------------------------------------------
// Initializes the IDirect3D8 and IDirect3DDevice8 interfaces
//----------------------------------------------------------------------------
HRESULT D3DUtilContext::InitDevice(const D3DDEVICE_CREATION_PARAMETERS& d3dcp, D3DPRESENT_PARAMETERS &d3dpp)
{
    // Make sure this device passes the user's requirements.
    FAIL_RETURN(TryConfirmDevice(d3dcp, d3dpp));

	if (Active) 
    {
        FAIL_TEST(Clear());
    }

    // if we are creating a fullscreen device, remember the window position.
    if (!d3dpp.Windowed)
    {
	    SaveWindowPosition(d3dcp, d3dpp);
    }

    // D3D fixes up things like BackBufferCount to reflect actual parameters.
    // Since we want to keep the defaults, we save our original settings.
    D3DPRESENT_PARAMETERS origpp = d3dpp;
	FAIL_RETURN_MSG(pD3D->CreateDevice(d3dcp.AdapterOrdinal, d3dcp.DeviceType, d3dcp.hFocusWindow, d3dcp.BehaviorFlags, &d3dpp, &pD3DDevice), "D3DUtilContext: CreateDevice Failed");
    d3dpp = origpp;

	SetCreationParameters(d3dcp);
	SetPresentParameters(d3dpp);

	FAIL_RETURN(InitRenderStates());                // Reset default state

    FAIL_RETURN(Activate(AT_Rebuild, !d3dpp.Windowed));

    return D3D_OK;
}

//----------------------------------------------------------------------------
// Reset
//----------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
HRESULT D3DUtilContext::ResetDevice(D3DPRESENT_PARAMETERS &newParams)
{
	if (!Active) FAIL_RETURN_MSG(E_FAIL, "D3DUtilContext: Cannot reset an inactive context. Call InitDevice first");

    FAIL_RETURN(TryConfirmDevice(D3DCP, newParams));

    // if we are swapping to a fullscreen device, save the window position
    if (D3DPP.Windowed && !newParams.Windowed)
    {
	    SaveWindowPosition(D3DCP, newParams);			// Save the window position if needed
    }
	
    if (Listener)
    {
        FAIL_RETURN(Listener->PrepareReset());      // Notify listeners to prepare for a reset
    }

    Active = false;

    // See if we are changing fullscreen/windowed mode
    bool togglingFullscren = D3DPP.Windowed != newParams.Windowed;

    // D3D fixes up things like BackBufferCount to reflect actual parameters.
    // Since we want to keep the defaults, we save our original settings.
    D3DPRESENT_PARAMETERS origpp = newParams;

    // Make sure out back buffer gets the proper size in windowed mode.
    // basically, don't allow width = height = 0 since when the device resets
    // the window will be the wrong size until we resize it manually (done
    // later in RestoreWindowPosition()).
	if (!D3DPP.Windowed && newParams.Windowed && (!newParams.BackBufferWidth || !newParams.BackBufferHeight))
	{
		newParams.BackBufferWidth = OldClientRect.right-OldClientRect.left;
		newParams.BackBufferHeight = OldClientRect.bottom-OldClientRect.top;
	}

	FAIL_RETURN_MSG(pD3DDevice->Reset(&newParams), "D3DUtilContext: Device Reset Failed");
    newParams = origpp;
    
	FAIL_RETURN(InitRenderStates());                // Reset default state

    if (newParams.Windowed && !D3DPP.Windowed)
    {
	    RestoreWindowPosition(newParams);			// Restore the window position if needed
    }

	SetPresentParameters(newParams);				// Remember the new presentation parameters

    FAIL_RETURN(Activate(AT_Reset, togglingFullscren));
    
    return D3D_OK;
}

//----------------------------------------------------------------------------
// Activate
//----------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
HRESULT D3DUtilContext::Activate(ActivationType type, bool togglingFullscreen)
{
    Active = true;

    if (Listener)
    {
        if (togglingFullscreen)
        {
            Listener->FullscreenToggle();
        }
        if (type == AT_Rebuild)
        {
            FAIL_RETURN(Listener->RebuildDevice());
        }
        FAIL_RETURN(Listener->Reset());
    }
    return D3D_OK;
}

//----------------------------------------------------------------------------
// Clear
//----------------------------------------------------------------------------
// Essentially resets the context to its default inactive state.
// Clears and releases all DX8 interfaces and internal data structures.
//----------------------------------------------------------------------------
HRESULT D3DUtilContext::Clear()
{
    if (Active) 
    {
        if (Listener)
        {
            FAIL_RETURN(Listener->PrepareReset());
            FAIL_RETURN(Listener->InvalidateDevice());
        }

        ULONG res = pD3DDevice->Release();
        if (res > 0)
        {
            AUXTRACE("D3DDevice refcount not zero when trying to clear device. Refcount = %d!!!", res);
        }
        pD3DDevice = NULL;
        SAFE_RELEASE(pD3DDevice);
        
        Active = false;

        if (!D3DPP.Windowed)
        {
            if (Listener)
            {
                Listener->FullscreenToggle();
            }
            RestoreWindowPosition(D3DPP);
        }
    }
    Init();
    return D3D_OK;
}

//----------------------------------------------------------------------------
// SaveWindowPosition
//----------------------------------------------------------------------------
// Save the window position before entering fullscreen mode
//----------------------------------------------------------------------------
void D3DUtilContext::SaveWindowPosition(const D3DDEVICE_CREATION_PARAMETERS &d3dcp, const D3DPRESENT_PARAMETERS &d3dpp)
{
	// AUXTRACE("Saving window position");
	GetWindowRect(d3dpp.hDeviceWindow ? d3dpp.hDeviceWindow : d3dcp.hFocusWindow, &OldWinRect);
	GetClientRect(d3dpp.hDeviceWindow ? d3dpp.hDeviceWindow : d3dcp.hFocusWindow, &OldClientRect);
}

//----------------------------------------------------------------------------
// RestoreWindowPosition
//----------------------------------------------------------------------------
// Restore the window position after leaving fullscreen mode
//----------------------------------------------------------------------------
void D3DUtilContext::RestoreWindowPosition(const D3DPRESENT_PARAMETERS &d3dpp)
{
	// AUXTRACE("Restoring window position");
	SetWindowPos(d3dpp.hDeviceWindow ? d3dpp.hDeviceWindow : D3DCP.hFocusWindow, HWND_NOTOPMOST, OldWinRect.left, OldWinRect.top, OldWinRect.right-OldWinRect.left, OldWinRect.bottom-OldWinRect.top, SWP_SHOWWINDOW);
}

//----------------------------------------------------------------------------
// TryConfirmDevice()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT 
D3DUtilContext::TryConfirmDevice(const D3DDEVICE_CREATION_PARAMETERS& d3dcp, const D3DPRESENT_PARAMETERS& d3dpp)
{
    // Always pass the reference rasterizer
    if (d3dcp.DeviceType == D3DDEVTYPE_REF) 
    {
        return D3D_OK;
    }

    D3DCAPS8 caps;
    FAIL_RETURN(pD3D->GetDeviceCaps(d3dcp.AdapterOrdinal, d3dcp.DeviceType, &caps));
    bool res = Listener->ConfirmDevice(caps, d3dcp, d3dpp);
    if (!res)
    {
        return D3DUTILERR_FAILEDCONFIRM;
    }
    return D3D_OK;
}

#pragma warning( pop )

