//============================================================================
// File          : Util.cpp
// 
// Content       : See header
// 
//============================================================================
// 
//============================================================================

//----------------------------------------------------------------------------
//-- Includes ----------------------------------------------------------------
//----------------------------------------------------------------------------
#include "stdafx.h"
#pragma warning( disable : 4786)
#include "Util.hpp"
#include <stdio.h>
#include <map>
#include <string>
#include <stdarg.h>

#pragma warning( disable : 4786)

//----------------------------------------------------------------------------
//-- Static Variables --------------------------------------------------------
//----------------------------------------------------------------------------

static const char *UNKNOWN_ENUM = "UNKNOWN ENUM";

//----------------------------------------------------------------------------
//-- Functions ---------------------------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// InstallPreProcessFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxAssert::PreProcessFunc
AuxAssert::InstallPreProcessFunc(PreProcessFunc func)
{
	PreProcessFunc oldFunc = _CurrentPreProcessFunc();
    _CurrentPreProcessFunc() = func;
    return oldFunc;
}

//----------------------------------------------------------------------------
// InstallPostProcessFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxAssert::PostProcessFunc
AuxAssert::InstallPostProcessFunc(PostProcessFunc func)
{
	PostProcessFunc oldFunc = _CurrentPostProcessFunc();
    _CurrentPostProcessFunc() = func;
    return oldFunc;
}

//----------------------------------------------------------------------------
// _GetPreProcessFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxAssert::PreProcessFunc&
AuxAssert::_CurrentPreProcessFunc()
{
	static PreProcessFunc func = NULL;
    return func;
}

//----------------------------------------------------------------------------
// _GetPostProcessFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxAssert::PostProcessFunc&
AuxAssert::_CurrentPostProcessFunc()
{
	static PostProcessFunc func = NULL;
    return func;
}

//----------------------------------------------------------------------------
// GetErrorTranslationFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxTrace::ErrorTranslationFunc
AuxTrace::GetErrorTranslationFunc()
{
	return _CurrentErrorTranslationFunc();
}

//----------------------------------------------------------------------------
// _CurrentErrorTranslationFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxTrace::ErrorTranslationFunc&
AuxTrace::_CurrentErrorTranslationFunc()
{
	static ErrorTranslationFunc func = DefaultErrorTranslationFunc;
    return func;
}

//----------------------------------------------------------------------------
// SetErrorTranslationFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
AuxTrace::ErrorTranslationFunc
AuxTrace::SetErrorTranslationFunc(ErrorTranslationFunc errFunc)
{
    ASSERT_MSG(errFunc != NULL, "!!! Error translation function cannot be NULL !!!");
	ErrorTranslationFunc oldFunc = _CurrentErrorTranslationFunc();
    _CurrentErrorTranslationFunc() = errFunc;
    return oldFunc;
}

//----------------------------------------------------------------------------
// DefaultErrorTranslationFunc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
const char*
AuxTrace::DefaultErrorTranslationFunc(HRESULT err)
{
	return DXGetErrorDescription8(err);
}

//-----------------------------------------------------------------------------
// Check
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
void __cdecl AuxAssert::Test( bool expr, const char* format, ... )
{
    if (expr) return;

    try
    {
        if (_CurrentPreProcessFunc() != NULL)
        {
            _CurrentPreProcessFunc()();
        }

        char strBuffer[1024];
        char *pBuf = strBuffer;

        pBuf += sprintf(strBuffer, "!!! Assertion failed (error follows) !!!\n%s(%d): ", _file, _line);

        va_list args;
        va_start(args, format);
        pBuf += vsprintf( pBuf, format, args );
        va_end(args);

        OutputDebugString( strBuffer );

        sprintf(pBuf, "\n\nPress Abort to quit, Retry to Debug, and Ignore to continue.");
        INT res = MessageBox(NULL, strBuffer, "Assertion Failed!", MB_ABORTRETRYIGNORE | MB_ICONERROR);
        switch (res)
        {
        case IDABORT:
            exit(-1);
            break;
        case IDRETRY:
            DebugBreak();
            break;
        case IDIGNORE:
        default:
            ;
        }

        if (_CurrentPostProcessFunc() != NULL)
        {
            _CurrentPostProcessFunc()();
        }
    }
    catch(...)
    {
        // Eat any and all exceptions. If we can't successfully log, then
        // we have bigger problems
    }
}

//-----------------------------------------------------------------------------
// AuxTrace
//-----------------------------------------------------------------------------
// Log debug output.
//-----------------------------------------------------------------------------
void __cdecl AuxTrace::Trace( const char* format, ... )
{
    try
    {
        char strBuffer[1024];
        char *pBuf = strBuffer;

        pBuf += sprintf(strBuffer, "%s(%d): ", _file, _line);

        va_list args;
        va_start(args, format);
        vsprintf( pBuf, format, args );
        va_end(args);
        strcat(strBuffer, "\n");

        OutputDebugString( strBuffer );
    }
    catch(...)
    {
        // Eat any and all exceptions. If we can't successfully log, then
        // we have bigger problems
    }
}

//----------------------------------------------------------------------------
// Rotate16v
//----------------------------------------------------------------------------
// Arbitrary-axis rotation. Special-cases axis-aligned rotations
//----------------------------------------------------------------------------
void MatrixRotate(D3DXMATRIX& M, float RadAng, const D3DXVECTOR3& Axis)
{
  float ca=cosf(RadAng),
        sa=sinf(RadAng);
  if (Axis[0]==1 && Axis[1]==0 && Axis[2]==0)  // ABOUT X-AXIS
  {
   M[0]=1; M[4]=0;  M[8]=0;   M[12]=0;
   M[1]=0; M[5]=ca; M[9]=-sa; M[13]=0;
   M[2]=0; M[6]=sa; M[10]=ca; M[14]=0;
   M[3]=0; M[7]=0;  M[11]=0;  M[15]=1;
  } else
  if (Axis[0]==0 && Axis[1]==1 && Axis[2]==0)  // ABOUT Y-AXIS
  {
   M[0]=ca;  M[4]=0; M[8]=sa;  M[12]=0;
   M[1]=0;   M[5]=1; M[9]=0;   M[13]=0;
   M[2]=-sa; M[6]=0; M[10]=ca; M[14]=0;
   M[3]=0;   M[7]=0; M[11]=0;  M[15]=1;
  } else
  if (Axis[0]==0 && Axis[1]==0 && Axis[2]==1)  // ABOUT Z-AXIS
  {
   M[0]=ca; M[4]=-sa; M[8]=0;  M[12]=0;
   M[1]=sa; M[5]=ca;  M[9]=0;  M[13]=0;
   M[2]=0;  M[6]=0;   M[10]=1; M[14]=0;
   M[3]=0;  M[7]=0;   M[11]=0; M[15]=1;
  }
  else                                      // ARBITRARY AXIS
  {
   float l = Axis[0]*Axis[0]+Axis[1]*Axis[1]+Axis[2]*Axis[2];
   float x, y, z;
   x=Axis[0],y=Axis[1],z=Axis[2];
   if (l > 1.0001f || l < 0.9999f && l!=0)
   {
     // needs normalization
     l=1.f/sqrtf(l);
     x*=l; y*=l; z*=l;
   }
   float x2=x*x, y2=y*y, z2=z*z;
   M[0]=x2+ca*(1-x2);            M[4]=(x*y)+ca*(-x*y)+sa*(-z); M[8]=(x*z)+ca*(-x*z)+sa*y;
   M[1]=(x*y)+ca*(-x*y)+sa*z;    M[5]=y2+ca*(1-y2);            M[9]=(y*z)+ca*(-y*z)+sa*(-x);
   M[2]=(x*z)+ca*(-x*z)+sa*(-y); M[6]=(y*z)+ca*(-y*z)+sa*x;    M[10]=z2+ca*(1-z2);
   M[12]=M[13]=M[14]=M[3]=M[7]=M[11]=0;
   M[15]=1;
  }
}

//-----------------------------------------------------------------------------
// Map to map D3DFORMATs to strings
//-----------------------------------------------------------------------------
typedef std::map<D3DFORMAT, std::string> FormatMapType;
typedef FormatMapType::value_type FormatPair;
static const FormatPair FormatPairs[] =
{
	FormatPair(D3DFMT_R8G8B8,std::string("D3DFMT_R8G8B8")),
	FormatPair(D3DFMT_A8R8G8B8,std::string("D3DFMT_A8R8G8B8")),
	FormatPair(D3DFMT_X8R8G8B8,std::string("D3DFMT_X8R8G8B8")),
	FormatPair(D3DFMT_R5G6B5,std::string("D3DFMT_R5G6B5")),
	FormatPair(D3DFMT_X1R5G5B5,std::string("D3DFMT_X1R5G5B5")),
	FormatPair(D3DFMT_A1R5G5B5,std::string("D3DFMT_A1R5G5B5")),
	FormatPair(D3DFMT_A4R4G4B4,std::string("D3DFMT_A4R4G4B4")),
	FormatPair(D3DFMT_R3G3B2,std::string("D3DFMT_R3G3B2")),
	FormatPair(D3DFMT_A8,std::string("D3DFMT_A8")),
	FormatPair(D3DFMT_A8R3G3B2,std::string("D3DFMT_A8R3G3B2")),
	FormatPair(D3DFMT_X4R4G4B4,std::string("D3DFMT_X4R4G4B4")),

	FormatPair(D3DFMT_A8P8,std::string("D3DFMT_A8P8")),    
	FormatPair(D3DFMT_P8,std::string("D3DFMT_P8")),

	FormatPair(D3DFMT_L8,std::string("D3DFMT_L8")),
	FormatPair(D3DFMT_A8L8,std::string("D3DFMT_A8L8")),
	FormatPair(D3DFMT_A4L4,std::string("D3DFMT_A4L4")),

	FormatPair(D3DFMT_V8U8,std::string("D3DFMT_V8U8")),
	FormatPair(D3DFMT_L6V5U5,std::string("D3DFMT_L6V5U5")),
	FormatPair(D3DFMT_X8L8V8U8,std::string("D3DFMT_X8L8V8U8")),
	FormatPair(D3DFMT_Q8W8V8U8,std::string("D3DFMT_Q8W8V8U8")),
	FormatPair(D3DFMT_V16U16,std::string("D3DFMT_V16U16")),
	FormatPair(D3DFMT_W11V11U10,std::string("D3DFMT_W11V11U10")),

	FormatPair(D3DFMT_UYVY,std::string("D3DFMT_UYVY")),
	FormatPair(D3DFMT_YUY2,std::string("D3DFMT_YUY2")),
	FormatPair(D3DFMT_DXT1,std::string("D3DFMT_DXT1")),
	FormatPair(D3DFMT_DXT2,std::string("D3DFMT_DXT2")),
	FormatPair(D3DFMT_DXT3,std::string("D3DFMT_DXT3")),
	FormatPair(D3DFMT_DXT4,std::string("D3DFMT_DXT4")),
	FormatPair(D3DFMT_DXT5,std::string("D3DFMT_DXT5")),

	FormatPair(D3DFMT_D16_LOCKABLE,std::string("D3DFMT_D16_LOCKABLE")),
	FormatPair(D3DFMT_D32,std::string("D3DFMT_D32")),
	FormatPair(D3DFMT_D15S1,std::string("D3DFMT_D15S1")),
	FormatPair(D3DFMT_D24S8,std::string("D3DFMT_D24S8")),
	FormatPair(D3DFMT_D16,std::string("D3DFMT_D16")),
	FormatPair(D3DFMT_D24X8,std::string("D3DFMT_D24X8")),
	FormatPair(D3DFMT_D24X4S4,std::string("D3DFMT_D24X4S4")),

	FormatPair(D3DFMT_VERTEXDATA,std::string("D3DFMT_VERTEXDATA")),
	FormatPair(D3DFMT_INDEX16,std::string("D3DFMT_INDEX16")),
	FormatPair(D3DFMT_INDEX32,std::string("D3DFMT_INDEX32")),
};
static FormatMapType FormatMap(&FormatPairs[0], &FormatPairs[sizeof(FormatPairs)/sizeof(FormatPair)]);

//-----------------------------------------------------------------------------
// Map to map strings to D3DFORMATs
//-----------------------------------------------------------------------------
typedef std::map<std::string, D3DFORMAT> FormatMapTypeReverse;
typedef FormatMapTypeReverse::value_type FormatPairReverse;
static const FormatPairReverse FormatPairReverses[] =
{
	FormatPairReverse(std::string("D3DFMT_R8G8B8"),D3DFMT_R8G8B8),
	FormatPairReverse(std::string("D3DFMT_A8R8G8B8"),D3DFMT_A8R8G8B8),
	FormatPairReverse(std::string("D3DFMT_X8R8G8B8"),D3DFMT_X8R8G8B8),
	FormatPairReverse(std::string("D3DFMT_R5G6B5"),D3DFMT_R5G6B5),
	FormatPairReverse(std::string("D3DFMT_X1R5G5B5"),D3DFMT_X1R5G5B5),
	FormatPairReverse(std::string("D3DFMT_A1R5G5B5"),D3DFMT_A1R5G5B5),
	FormatPairReverse(std::string("D3DFMT_A4R4G4B4"),D3DFMT_A4R4G4B4),
	FormatPairReverse(std::string("D3DFMT_R3G3B2"),D3DFMT_R3G3B2),
	FormatPairReverse(std::string("D3DFMT_A8"),D3DFMT_A8),
	FormatPairReverse(std::string("D3DFMT_A8R3G3B2"),D3DFMT_A8R3G3B2),
	FormatPairReverse(std::string("D3DFMT_X4R4G4B4"),D3DFMT_X4R4G4B4),

	FormatPairReverse(std::string("D3DFMT_A8P8"),D3DFMT_A8P8),    
	FormatPairReverse(std::string("D3DFMT_P8"),D3DFMT_P8),

	FormatPairReverse(std::string("D3DFMT_L8"),D3DFMT_L8),
	FormatPairReverse(std::string("D3DFMT_A8L8"),D3DFMT_A8L8),
	FormatPairReverse(std::string("D3DFMT_A4L4"),D3DFMT_A4L4),

	FormatPairReverse(std::string("D3DFMT_V8U8"),D3DFMT_V8U8),
	FormatPairReverse(std::string("D3DFMT_L6V5U5"),D3DFMT_L6V5U5),
	FormatPairReverse(std::string("D3DFMT_X8L8V8U8"),D3DFMT_X8L8V8U8),
	FormatPairReverse(std::string("D3DFMT_Q8W8V8U8"),D3DFMT_Q8W8V8U8),
	FormatPairReverse(std::string("D3DFMT_V16U16"),D3DFMT_V16U16),
	FormatPairReverse(std::string("D3DFMT_W11V11U10"),D3DFMT_W11V11U10),

	FormatPairReverse(std::string("D3DFMT_UYVY"),D3DFMT_UYVY),
	FormatPairReverse(std::string("D3DFMT_YUY2"),D3DFMT_YUY2),
	FormatPairReverse(std::string("D3DFMT_DXT1"),D3DFMT_DXT1),
	FormatPairReverse(std::string("D3DFMT_DXT2"),D3DFMT_DXT2),
	FormatPairReverse(std::string("D3DFMT_DXT3"),D3DFMT_DXT3),
	FormatPairReverse(std::string("D3DFMT_DXT4"),D3DFMT_DXT4),
	FormatPairReverse(std::string("D3DFMT_DXT5"),D3DFMT_DXT5),

	FormatPairReverse(std::string("D3DFMT_D16_LOCKABLE"),D3DFMT_D16_LOCKABLE),
	FormatPairReverse(std::string("D3DFMT_D32"),D3DFMT_D32),
	FormatPairReverse(std::string("D3DFMT_D15S1"),D3DFMT_D15S1),
	FormatPairReverse(std::string("D3DFMT_D24S8"),D3DFMT_D24S8),
	FormatPairReverse(std::string("D3DFMT_D16"),D3DFMT_D16),
	FormatPairReverse(std::string("D3DFMT_D24X8"),D3DFMT_D24X8),
	FormatPairReverse(std::string("D3DFMT_D24X4S4"),D3DFMT_D24X4S4),

	FormatPairReverse(std::string("D3DFMT_VERTEXDATA"),D3DFMT_VERTEXDATA),
	FormatPairReverse(std::string("D3DFMT_INDEX16"),D3DFMT_INDEX16),
	FormatPairReverse(std::string("D3DFMT_INDEX32"),D3DFMT_INDEX32),
};
static FormatMapTypeReverse FormatMapReverse(&FormatPairReverses[0], &FormatPairReverses[sizeof(FormatPairReverses)/sizeof(FormatPairReverse)]);

//-----------------------------------------------------------------------------
// Map to map D3DMULTISAMPLE_TYPEs to strings
//-----------------------------------------------------------------------------
typedef std::map<D3DMULTISAMPLE_TYPE, std::string> MultisampleMapType;
typedef MultisampleMapType::value_type MultisamplePair;
static const MultisamplePair MultisamplePairs[] =
{
    MultisamplePair(D3DMULTISAMPLE_NONE,std::string("D3DMULTISAMPLE_NONE")),
    MultisamplePair(D3DMULTISAMPLE_2_SAMPLES,std::string("D3DMULTISAMPLE_2_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_3_SAMPLES,std::string("D3DMULTISAMPLE_3_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_4_SAMPLES,std::string("D3DMULTISAMPLE_4_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_5_SAMPLES,std::string("D3DMULTISAMPLE_5_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_6_SAMPLES,std::string("D3DMULTISAMPLE_6_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_7_SAMPLES,std::string("D3DMULTISAMPLE_7_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_8_SAMPLES,std::string("D3DMULTISAMPLE_8_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_9_SAMPLES,std::string("D3DMULTISAMPLE_9_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_10_SAMPLES,std::string("D3DMULTISAMPLE_10_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_11_SAMPLES,std::string("D3DMULTISAMPLE_11_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_12_SAMPLES,std::string("D3DMULTISAMPLE_12_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_13_SAMPLES,std::string("D3DMULTISAMPLE_13_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_14_SAMPLES,std::string("D3DMULTISAMPLE_14_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_15_SAMPLES,std::string("D3DMULTISAMPLE_15_SAMPLES")),
    MultisamplePair(D3DMULTISAMPLE_16_SAMPLES,std::string("D3DMULTISAMPLE_16_SAMPLES")),
};
static MultisampleMapType MultisampleMap(&MultisamplePairs[0], &MultisamplePairs[sizeof(MultisamplePairs)/sizeof(MultisamplePair)]);

//-----------------------------------------------------------------------------
// Map to map strings to D3DMULTISAMPLE_TYPEs
//-----------------------------------------------------------------------------
typedef std::map<std::string, D3DMULTISAMPLE_TYPE> MultisampleMapTypeReverse;
typedef MultisampleMapTypeReverse::value_type MultisamplePairReverse;
static const MultisamplePairReverse MultisamplePairReverses[] =
{
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_NONE"),D3DMULTISAMPLE_NONE),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_2_SAMPLES"),D3DMULTISAMPLE_2_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_3_SAMPLES"),D3DMULTISAMPLE_3_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_4_SAMPLES"),D3DMULTISAMPLE_4_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_5_SAMPLES"),D3DMULTISAMPLE_5_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_6_SAMPLES"),D3DMULTISAMPLE_6_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_7_SAMPLES"),D3DMULTISAMPLE_7_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_8_SAMPLES"),D3DMULTISAMPLE_8_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_9_SAMPLES"),D3DMULTISAMPLE_9_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_10_SAMPLES"),D3DMULTISAMPLE_10_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_11_SAMPLES"),D3DMULTISAMPLE_11_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_12_SAMPLES"),D3DMULTISAMPLE_12_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_13_SAMPLES"),D3DMULTISAMPLE_13_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_14_SAMPLES"),D3DMULTISAMPLE_14_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_15_SAMPLES"),D3DMULTISAMPLE_15_SAMPLES),
    MultisamplePairReverse(std::string("D3DMULTISAMPLE_16_SAMPLES"),D3DMULTISAMPLE_16_SAMPLES),
};
static MultisampleMapTypeReverse MultisampleMapReverse(&MultisamplePairReverses[0], &MultisamplePairReverses[sizeof(MultisamplePairReverses)/sizeof(MultisamplePairReverse)]);

//-----------------------------------------------------------------------------
// Map to map D3DDEVTYPEs to strings
//-----------------------------------------------------------------------------
typedef std::map<D3DDEVTYPE, std::string> DeviceMapType;
typedef DeviceMapType::value_type DevicePair;
static const DevicePair DevicePairs[] =
{
    DevicePair(D3DDEVTYPE_HAL,std::string("D3DDEVTYPE_HAL")),
    DevicePair(D3DDEVTYPE_REF,std::string("D3DDEVTYPE_REF")),
};
static DeviceMapType DeviceMap(&DevicePairs[0], &DevicePairs[sizeof(DevicePairs)/sizeof(DevicePair)]);

//-----------------------------------------------------------------------------
// Map to map strings to D3DDEVTYPEs
//-----------------------------------------------------------------------------
typedef std::map<std::string, D3DDEVTYPE> DeviceMapTypeReverse;
typedef DeviceMapTypeReverse::value_type DevicePairReverse;
static const DevicePairReverse DevicePairReverses[] =
{
    DevicePairReverse(std::string("D3DDEVTYPE_HAL"),D3DDEVTYPE_HAL),
    DevicePairReverse(std::string("D3DDEVTYPE_REF"),D3DDEVTYPE_REF),
};
static DeviceMapTypeReverse DeviceMapReverse(&DevicePairReverses[0], &DevicePairReverses[sizeof(DevicePairReverses)/sizeof(DevicePairReverse)]);

//-----------------------------------------------------------------------------
// EnumToString
//-----------------------------------------------------------------------------
// Several routines to translate DX8 enums to strings.
//-----------------------------------------------------------------------------
const char * EnumToString(D3DFORMAT format)
{
	FormatMapType::iterator iter = FormatMap.find(format);
	if (iter != FormatMap.end()) return iter->second.c_str(); else return UNKNOWN_ENUM;
}

//-----------------------------------------------------------------------------
// StringToEnum
//-----------------------------------------------------------------------------
// Several routines to translate strings to DX8 enums.
//-----------------------------------------------------------------------------
D3DFORMAT StringToD3DFORMAT(const char *str)
{
	FormatMapTypeReverse::iterator iter = FormatMapReverse.find(str);
	if (iter != FormatMapReverse.end()) return iter->second; else return D3DFMT_UNKNOWN;
}

//-----------------------------------------------------------------------------
// EnumToString
//-----------------------------------------------------------------------------
// Several routines to translate DX8 enums to strings.
//-----------------------------------------------------------------------------
const char * EnumToString(D3DMULTISAMPLE_TYPE type)
{
	MultisampleMapType::iterator iter = MultisampleMap.find(type);
	if (iter != MultisampleMap.end()) return iter->second.c_str(); else return UNKNOWN_ENUM;
}

//-----------------------------------------------------------------------------
// StringToEnum
//-----------------------------------------------------------------------------
// Several routines to translate strings to DX8 enums.
//-----------------------------------------------------------------------------
D3DMULTISAMPLE_TYPE StringToD3DMULTISAMPLE_TYPE(const char *str)
{
	MultisampleMapTypeReverse::iterator iter = MultisampleMapReverse.find(str);
	if (iter != MultisampleMapReverse.end()) return iter->second; else return D3DMULTISAMPLE_NONE;
}


//-----------------------------------------------------------------------------
// EnumToString
//-----------------------------------------------------------------------------
// Several routines to translate DX8 enums to strings.
//-----------------------------------------------------------------------------
const char * EnumToString(D3DDEVTYPE type)
{
	DeviceMapType::iterator iter = DeviceMap.find(type);
	if (iter != DeviceMap.end()) return iter->second.c_str(); else return UNKNOWN_ENUM;
}

//-----------------------------------------------------------------------------
// StringToEnum
//-----------------------------------------------------------------------------
// Several routines to translate strings to DX8 enums.
//-----------------------------------------------------------------------------
D3DDEVTYPE StringToD3DDEVTYPE(const char *str)
{
	DeviceMapTypeReverse::iterator iter = DeviceMapReverse.find(str);
	if (iter != DeviceMapReverse.end()) return iter->second; else return D3DDEVTYPE_SW;
}
