//-----------------------------------------------------------------------------
// D3DUtilEnumeration.hpp
//-----------------------------------------------------------------------------
// Enumerates and creates a list of all the devices and modes on the system.
//
//-----------------------------------------------------------------------------
#ifndef _D3DUtilEnumeration_hpp_
#define _D3DUtilEnumeration_hpp_
#pragma once

//-----------------------------------------------------------------------------
// #includes
//-----------------------------------------------------------------------------
#include "Util.hpp"
#include <vector>
#include <algorithm>

//-----------------------------------------------------------------------------
// Forwards
//-----------------------------------------------------------------------------
class D3DUtilAdapterInfo;
class D3DUtilDisplayModeInfo;
class D3DUtilDeviceInfo;
class D3DUtilAdapterModeInfo;
class D3DUtilBackBufferInfo;
class D3DUtilDepthInfo;
class D3DUtilMultisampleInfo;

typedef std::vector<D3DUtilAdapterInfo>     D3DUtilAdapterInfoList;
typedef std::vector<D3DUtilDisplayModeInfo> D3DUtilDisplayModeInfoList;
typedef std::vector<D3DUtilDeviceInfo>      D3DUtilDeviceInfoList;
typedef std::vector<D3DUtilAdapterModeInfo> D3DUtilAdapterModeInfoList;
typedef std::vector<D3DUtilBackBufferInfo>  D3DUtilBackBufferInfoList;
typedef std::vector<D3DUtilDepthInfo>       D3DUtilDepthInfoList;
typedef std::vector<D3DUtilMultisampleInfo> D3DUtilMultisampleInfoList;

//-----------------------------------------------------------------------------
// Maps certain enums to contiguous arrays so they can be iterated over
//-----------------------------------------------------------------------------
const D3DFORMAT BackBufferFormats[] = {
    D3DFMT_A2R10G10B10,
    D3DFMT_X8R8G8B8, 
    D3DFMT_A8R8G8B8, 
    D3DFMT_X1R5G5B5, 
    D3DFMT_A1R5G5B5,
    D3DFMT_R5G6B5, 
};
const int BackBufferFormatCount = sizeof(BackBufferFormats) / sizeof(D3DFORMAT);

/*
const D3DFORMAT Formats[] = {
    D3DFMT_R8G8B8,
	D3DFMT_A8R8G8B8,
    D3DFMT_X8R8G8B8,
    D3DFMT_R5G6B5,
    D3DFMT_X1R5G5B5,
    D3DFMT_A1R5G5B5,
    D3DFMT_A4R4G4B4,
    D3DFMT_R3G3B2,
    D3DFMT_A8,
    D3DFMT_A8R3G3B2,
    D3DFMT_X4R4G4B4,

    D3DFMT_A8P8,    
    D3DFMT_P8,

    D3DFMT_L8,
    D3DFMT_A8L8,
    D3DFMT_A4L4,

    D3DFMT_V8U8,
    D3DFMT_L6V5U5,
    D3DFMT_X8L8V8U8,
    D3DFMT_Q8W8V8U8,
    D3DFMT_V16U16,

    D3DFMT_UYVY,
    D3DFMT_YUY2,
    D3DFMT_DXT1,
    D3DFMT_DXT2,
    D3DFMT_DXT3,
    D3DFMT_DXT4,
    D3DFMT_DXT5,

    D3DFMT_D16_LOCKABLE,
    D3DFMT_D32,
    D3DFMT_D15S1,
    D3DFMT_D24S8,
    D3DFMT_D16,
    D3DFMT_D24X8,
    D3DFMT_D24X4S4,

    D3DFMT_VERTEXDATA,
    D3DFMT_INDEX16,
    D3DFMT_INDEX32,
};
const int FormatCount = sizeof(Formats) / sizeof(D3DFORMAT);
*/

const D3DDEVTYPE DeviceTypes[] = {
	D3DDEVTYPE_HAL,
	D3DDEVTYPE_REF,
};
const int DeviceTypeCount = sizeof(DeviceTypes) / sizeof(D3DFORMAT);

const D3DFORMAT DepthFormats[] = {
    D3DFMT_D24S8,
    D3DFMT_D15S1,
    D3DFMT_D32,
    D3DFMT_D24X8,
    D3DFMT_D24X4S4,
    D3DFMT_D16,
    D3DFMT_D16_LOCKABLE,
	D3DFMT_UNKNOWN,  // signifies that we are not using a depth buffer when enumerating.
};
const int DepthFormatCount = sizeof(DepthFormats) / sizeof(D3DFORMAT);

const D3DMULTISAMPLE_TYPE MultisampleTypes[] = {
	D3DMULTISAMPLE_NONE,
    D3DMULTISAMPLE_NONMASKABLE,
	D3DMULTISAMPLE_2_SAMPLES,
	D3DMULTISAMPLE_3_SAMPLES,
	D3DMULTISAMPLE_4_SAMPLES,
	D3DMULTISAMPLE_5_SAMPLES,
	D3DMULTISAMPLE_6_SAMPLES,
	D3DMULTISAMPLE_7_SAMPLES,
	D3DMULTISAMPLE_8_SAMPLES,
	D3DMULTISAMPLE_9_SAMPLES,
	D3DMULTISAMPLE_10_SAMPLES,
	D3DMULTISAMPLE_11_SAMPLES,
	D3DMULTISAMPLE_12_SAMPLES,
	D3DMULTISAMPLE_13_SAMPLES,
	D3DMULTISAMPLE_14_SAMPLES,
	D3DMULTISAMPLE_15_SAMPLES,
	D3DMULTISAMPLE_16_SAMPLES,
};
const int MultisampleCount = sizeof(MultisampleTypes) / sizeof(D3DMULTISAMPLE_TYPE);

//-----------------------------------------------------------------------------
// D3DUtilDepthInfo
//-----------------------------------------------------------------------------
class D3DUtilMultisampleInfo
{
public:
	D3DMULTISAMPLE_TYPE Type;
    DWORD QualityLevels;
};

//-----------------------------------------------------------------------------
// D3DUtilDepthInfo
//-----------------------------------------------------------------------------
class D3DUtilDepthInfo
{
public:
	const D3DUtilMultisampleInfo* FindMultisample(D3DMULTISAMPLE_TYPE type) const
	{
        D3DUtilMultisampleInfoList::const_iterator res = std::find_if(MultisampleTypes.begin(), MultisampleTypes.end(), MultisampleFinder(type));
		return res == MultisampleTypes.end() ? NULL : &(*res);
	}

	D3DFORMAT Format;
	D3DUtilMultisampleInfoList MultisampleTypes;
private:
	class MultisampleFinder
	{
	public:
		MultisampleFinder(D3DMULTISAMPLE_TYPE type) : Type(type) {}
		bool operator()(const D3DUtilMultisampleInfo& multi)
		{
			return multi.Type == Type;
		}
	private:
		MultisampleFinder() : Type(D3DMULTISAMPLE_NONE) {}
		D3DMULTISAMPLE_TYPE Type;

	};
};

//-----------------------------------------------------------------------------
// D3DUtilBackBufferInfo
//-----------------------------------------------------------------------------
class D3DUtilBackBufferInfo
{
public:
	const D3DUtilDepthInfo* FindDepth(D3DFORMAT format) const
	{
		D3DUtilDepthInfoList::const_iterator res = std::find_if(DepthBuffers.begin(), DepthBuffers.end(), DepthFinder(format));
		return res == DepthBuffers.end() ? NULL : &(*res);
	}

	D3DFORMAT Format;
    BOOL HardwareConversion;
	D3DUtilDepthInfoList DepthBuffers;
private:
	class DepthFinder
	{
	public:
		DepthFinder(D3DFORMAT format) : Format(format) {}
		bool operator()(const D3DUtilDepthInfo &DepthInfo)
		{
			return DepthInfo.Format == Format;
		}
	private:
		DepthFinder() : Format(D3DFMT_UNKNOWN) {}
		D3DFORMAT Format;

	};
};

//-----------------------------------------------------------------------------
// D3DUtilAdapterModeInfo
//-----------------------------------------------------------------------------
class D3DUtilAdapterModeInfo
{
public:
    D3DFORMAT Mode;
    BOOL Windowed;
    D3DUtilBackBufferInfoList BackBuffers;

    const D3DUtilBackBufferInfo *FindBackBuffer(D3DFORMAT format) const
	{
		D3DUtilBackBufferInfoList::const_iterator res = std::find_if(BackBuffers.begin(), BackBuffers.end(), BackBufferFinder(format));
		return res == BackBuffers.end() ? NULL : &(*res);
	}
private:
	class BackBufferFinder
	{
	public:
		BackBufferFinder(D3DFORMAT format) : Format(format) {}
		bool operator()(const D3DUtilBackBufferInfo &backBufferInfo)
		{
			return backBufferInfo.Format == Format;
		}
	private:
		BackBufferFinder() : Format(D3DFMT_UNKNOWN) {}
		D3DFORMAT Format;

	};
};

//-----------------------------------------------------------------------------
// D3DUtilDeviceInfo
//-----------------------------------------------------------------------------
class D3DUtilDeviceInfo
{
public:
	D3DDEVTYPE DeviceType;
	D3DCAPS9 DeviceCaps;
	D3DUtilAdapterModeInfoList AdapterModes;

    const D3DUtilAdapterModeInfo *FindAdapterMode(D3DFORMAT mode, BOOL windowed) const
	{
		D3DUtilAdapterModeInfoList::const_iterator res = std::find_if(AdapterModes.begin(), AdapterModes.end(), AdapterModeFinder(mode, windowed));
		return res == AdapterModes.end() ? NULL : &(*res);
	}
private:
	class AdapterModeFinder
	{
	public:
		AdapterModeFinder(D3DFORMAT mode, BOOL windowed) : Mode(mode), Windowed(windowed) {}
		bool operator()(const D3DUtilAdapterModeInfo &adapterModeInfo)
		{
			return (adapterModeInfo.Mode == Mode) && (adapterModeInfo.Windowed == Windowed);
		}
	private:
		AdapterModeFinder() : Mode(D3DFMT_UNKNOWN) {}
		D3DFORMAT Mode;
        BOOL Windowed;

	};
};

inline
bool LessThan(const D3DDISPLAYMODE& ModeA, const D3DDISPLAYMODE& ModeB)
{
		if (ModeA.Width < ModeB.Width) return true;
		else if (ModeA.Width == ModeB.Width)
			if (ModeA.Height < ModeB.Height) return true;
			else if (ModeA.Height == ModeB.Height)
				if (int(ModeA.Format) > int(ModeB.Format)) return true;
				else if (int(ModeA.Format) == int(ModeB.Format))
					if (ModeA.RefreshRate < ModeB.RefreshRate) return true;
        return false;
}

//-----------------------------------------------------------------------------
// D3DUtilDisplayModeInfo
//-----------------------------------------------------------------------------
class D3DUtilDisplayModeInfo
{
public:
    D3DFORMAT DisplayModeFormat;
    std::vector<D3DDISPLAYMODE> Modes; // sorted using LessThan (above)
};

//-----------------------------------------------------------------------------
// D3DUtilAdapterInfo
//-----------------------------------------------------------------------------
class D3DUtilAdapterInfo
{
public:
	const D3DUtilDeviceInfo* FindDevice(D3DDEVTYPE type) const
	{
		D3DUtilDeviceInfoList::const_iterator res = std::find_if(Devices.begin(), Devices.end(), DevTypeFinder(type));
		return res == Devices.end() ? NULL : &(*res);
	};

	const D3DUtilDisplayModeInfo* FindDisplayModes(D3DFORMAT format) const
	{
		D3DUtilDisplayModeInfoList::const_iterator res = std::find_if(DisplayModes.begin(), DisplayModes.end(), DisplayModeFinder(format));
		return res == DisplayModes.end() ? NULL : &(*res);
	};

	char Driver[MAX_DEVICE_IDENTIFIER_STRING];
	char Description[MAX_DEVICE_IDENTIFIER_STRING];
    char DeviceName[32];
	D3DUtilDeviceInfoList Devices;
	D3DUtilDisplayModeInfoList DisplayModes; // One list for each back buffer format.

private:
	class DevTypeFinder
	{
	public:
		DevTypeFinder(D3DDEVTYPE type) : DevType(type) {}
		bool operator()(const D3DUtilDeviceInfo &devInfo)
		{
			return devInfo.DeviceType == DevType;
		}
	private:
		DevTypeFinder() : DevType(D3DDEVTYPE_HAL) {}
		D3DDEVTYPE DevType;

	};

	class DisplayModeFinder
	{
	public:
		DisplayModeFinder(D3DFORMAT format) : Format(format) {}
		bool operator()(const D3DUtilDisplayModeInfo &modeInfo)
		{
			return modeInfo.DisplayModeFormat == Format;
		}
	private:
		DisplayModeFinder() {}
		D3DFORMAT Format;
	};
};

//-----------------------------------------------------------------------------
// D3DUtilEnumeration
//-----------------------------------------------------------------------------
class D3DUtilEnumeration
{
public:
	static const D3DUtilAdapterInfoList &GetAdapterList();
private:
	static void BuildAdapterList(IDirect3D9 *pD3D);
	static void BuildDisplayModeList(int adapter, IDirect3D9 *pD3D);
	static void BuildDeviceList(int adapter, IDirect3D9 *pD3D);
	static void BuildAdapterModeList(int adapter, D3DUtilDeviceInfo &deviceInfo, IDirect3D9 *pD3D);
	static void BuildBackBufferList(int adapter, D3DDEVTYPE deviceType, D3DUtilAdapterModeInfo &adapterModeInfo, IDirect3D9 *pD3D);
	static void BuildDepthBufferList(int adapter, D3DDEVTYPE deviceType, D3DFORMAT adapterMode, BOOL windowed, D3DUtilBackBufferInfo &backBufferInfo, IDirect3D9 *pD3D);
	static void BuildMultisampleList(int adapter, D3DDEVTYPE deviceType, D3DFORMAT adapterMode, BOOL windowed, D3DFORMAT backBufferFormat, D3DUtilDepthInfo &depthInfo, IDirect3D9 *pD3D);

	static D3DUtilAdapterInfoList AdapterList;
};

#endif
