//-----------------------------------------------------------------------------
// UtilCamRecord.cpp
//-----------------------------------------------------------------------------
// Camera record/playback tools.
//
//-----------------------------------------------------------------------------
#include "stdafx.h"
#include "UtilCamRecord.hpp"
#include <float.h>
#include <windows.h>
#include <mmsystem.h>

UtilCamRecord::UtilCamRecord()
{
	Active = false;
	pCamera = NULL;
	Vec3Set(OldView, FLT_MAX, FLT_MAX, FLT_MAX);
	Vec3Set(OldPos, FLT_MAX, FLT_MAX, FLT_MAX);
}

UtilCamRecord::~UtilCamRecord()
{
	Close();
}

bool UtilCamRecord::BeginRecord(const char *Filename, UtilCamera *camera)
{
    file = new std::fstream(Filename, std::ios_base::out | std::ios_base::binary);
	if (!file->is_open()) return false;
	if (!camera) return false;

	pCamera = camera;
	Active = true;
	return true;
}

bool UtilCamRecord::RecordStep()
{
	if (!Active) return false;
	if (file->eof()) return false;

	if (pCamera->GetPosition() == OldPos && pCamera->GetView() == OldView) return true;

	file->write(reinterpret_cast<const char*>(&pCamera->GetPosition().x), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetPosition().y), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetPosition().z), sizeof(float));

	file->write(reinterpret_cast<const char*>(&pCamera->GetView().x), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetView().y), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetView().z), sizeof(float));

	file->write(reinterpret_cast<const char*>(&pCamera->GetUp().x), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetUp().y), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetUp().z), sizeof(float));

	file->write(reinterpret_cast<const char*>(&pCamera->GetSide().x), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetSide().y), sizeof(float));
	file->write(reinterpret_cast<const char*>(&pCamera->GetSide().z), sizeof(float));

	OldPos = pCamera->GetPosition();
	OldView = pCamera->GetView();

	if (file->fail()) return false;
	return true;
}

bool UtilCamRecord::Close()
{
	if (Active)
	{
		file->close();
		delete file; file = NULL;
		pCamera = NULL;
		Vec3Set(OldPos, FLT_MAX, FLT_MAX, FLT_MAX);
		Vec3Set(OldView, FLT_MAX, FLT_MAX, FLT_MAX);
		Active = false;
	}
	return true;
}

UtilCamPlay::UtilCamPlay()
{
	Active = false;
	pCamera = NULL;
	NumFrames = 0;
	TimePlayed = 0;
}

UtilCamPlay::~UtilCamPlay()
{
	Close();
}

bool UtilCamPlay::OpenForPlayback(const char *Filename, UtilCamera *camera)
{
    file = new std::fstream(Filename, std::ios_base::in | std::ios_base::binary);
	if (!file->is_open() || file->bad() || file->eof()) return false;
	if (!camera) return false;

	NumFrames = 0;
	TimePlayed = timeGetTime();
	pCamera = camera;
	Active = true;
	return true;
}

bool UtilCamPlay::PlayStep(int numSteps)
{
	if (!Active) return false;
	if (file->eof()) return false;

	D3DXVECTOR3 p,v,s,u;

	file->read(reinterpret_cast<char*>(&p.x), sizeof(float));
	file->read(reinterpret_cast<char*>(&p.y), sizeof(float));
	file->read(reinterpret_cast<char*>(&p.z), sizeof(float));

	file->read(reinterpret_cast<char*>(&v.x), sizeof(float));
	file->read(reinterpret_cast<char*>(&v.y), sizeof(float));
	file->read(reinterpret_cast<char*>(&v.z), sizeof(float));

	file->read(reinterpret_cast<char*>(&u.x), sizeof(float));
	file->read(reinterpret_cast<char*>(&u.y), sizeof(float));
	file->read(reinterpret_cast<char*>(&u.z), sizeof(float));

	file->read(reinterpret_cast<char*>(&s.x), sizeof(float));
	file->read(reinterpret_cast<char*>(&s.y), sizeof(float));
	file->read(reinterpret_cast<char*>(&s.z), sizeof(float));
	
	if (file->fail() || file->eof()) return false;

	pCamera->SetPosition(p);
	pCamera->SetViewSideUp(v,s,u);
	NumFrames++;
	return true;
}

bool UtilCamPlay::Close()
{
	if (Active)
	{
		TimePlayed = timeGetTime()-TimePlayed;
		file->close();
		delete file; file = NULL;
		pCamera = NULL;
		Active = false;
	}
	return true;
}

void UtilCamPlay::GetLastPlayStats(unsigned* numFrames, unsigned* timeMS)
{
	if (numFrames) *numFrames = NumFrames;
	if (timeMS) *timeMS = TimePlayed;
}
