//-----------------------------------------------------------------------------
// UtilCamRecord.hpp
//-----------------------------------------------------------------------------
// Camera record/playback tools.
//
//-----------------------------------------------------------------------------
#ifndef UtilCamRecord__Included
#define UtilCamRecord__Included
#pragma once

#include "Util.hpp"
#include "UtilCamera.hpp"
#include <fstream>

//-----------------------------------------------------------------------------
// UtilCamRecord
//-----------------------------------------------------------------------------
// Class to record camera movements.
//-----------------------------------------------------------------------------
class UtilCamRecord
{
public:
	UtilCamRecord();
	~UtilCamRecord();

	bool BeginRecord(const char *Filename, UtilCamera *camera);
	bool RecordStep();

	bool Close();

	bool IsActive() const                  { return Active; }
	void SetCamera(UtilCamera *camera)         { if (Active) pCamera = camera; }
protected:
	bool Active;

	UtilCamera *pCamera;
    std::fstream *file;

	D3DXVECTOR3 OldPos, OldView;
};

//-----------------------------------------------------------------------------
// UtilCamPlay
//-----------------------------------------------------------------------------
// Class to playback camera recordings.
//-----------------------------------------------------------------------------
class UtilCamPlay
{
public:
	UtilCamPlay();
	~UtilCamPlay();

	bool OpenForPlayback(const char *Filename, UtilCamera *camera);
	bool PlayStep(int numSteps);

	bool Close();

	void GetLastPlayStats(unsigned* numFrames=NULL, unsigned* timeMS=NULL);

	bool IsActive() const { return Active; }
	void SetCamera(UtilCamera *camera) { if (Active) pCamera = camera; }
protected:
	bool Active;

	UtilCamera *pCamera;
    std::fstream *file;

	unsigned int NumFrames;
	unsigned int TimePlayed;
};

#endif
