//============================================================================
// File          : d3d8Expanded.cpp
// 
// Content       : See header
// 
//============================================================================
// 
//============================================================================

//----------------------------------------------------------------------------
//-- Includes ----------------------------------------------------------------
//----------------------------------------------------------------------------

#include "d3d8Expanded.hpp"
#include "resourcesD3D8/resource.h"
#include <vector>

//----------------------------------------------------------------------------
//-- Static Variables --------------------------------------------------------
//----------------------------------------------------------------------------
const DWORD finalFVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX2 | D3DFVF_TEXCOORDSIZE2(0) | D3DFVF_TEXCOORDSIZE3(1);

const DWORD INVALID_SHADER_HANDLE = 0xFFFFFFFF;

D3DXCOLOR MatDiffuse(0.0f,0.8f,1.0f,1);
//D3DXCOLOR MatDiffuse(0.0f,0.0f,0.0f,1);
D3DXCOLOR MatAmbientEmissive(0.1f,0.1f,0.1f,1.f);
//D3DXCOLOR MatAmbientEmissive(0.0f,0.0f,0.0f,1.f);
D3DXCOLOR MatSpecular(0.3f,0.9f,0.7f,1);
//D3DXCOLOR MatSpecular(0.0f,0.0f,0.0f,1);
D3DXVECTOR4 MatPower(16.f,0,0,1);

const char* STR_EARTH_MAP = "..\\common\\media\\earth.png";
const char* STR_EARTH_BUMP_MAP = "..\\common\\media\\earth_bump.png";
const char* STR_CHAMELEON = "..\\common\\media\\chameleon.x";

CGprofile PixelProfile = CG_PROFILE_PS_1_1;
CGprofile VertexProfile = CG_PROFILE_VS_1_1;

//----------------------------------------------------------------------------
//-- Functions ---------------------------------------------------------------
//----------------------------------------------------------------------------

// outputs errors one line at a time so we don't overflow OutputDebugString's capabilities
void OutputErrors(const char* errs)
{
    // Output the listing to the debug console
    std::vector<char> l(strlen(errs)+1);
    std::copy(errs, errs+l.size(), l.begin());
    const char* t = strtok(&l.front(), "\n");
    do {OutputDebugString(t); OutputDebugString("\n");} while (t = strtok(NULL, "\n"));
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    d3d8Expanded test;

    if( FAILED( test.WindowCreate( hInst, 
        "D3D8 Expanded Interface Cg shader app - Press H for help", 
        512, 512, 
        IDR_MENU1, 
        IDI_MAIN_ICON,
        IDR_MAIN_ACCEL)))
      return 0;

    HRESULT hres;
    FAIL_ASSERT_MSG(hres=test.Run(AUTOMATIC_DEVICE_CREATION), "The App has encountered as fatal error:");
    return hres;
}

//----------------------------------------------------------------------------
// cgErrorCallback()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
void __cdecl cgErrorCallback()
{
    CGerror err = cgGetError();
    const char* str = cgD3D8TranslateCGerror(err);
    if (err == cgD3D8Failed)
    {
        HRESULT hres = cgD3D8GetLastError();
        const char* errStr = cgD3D8TranslateHRESULT(hres);
    } else if (err == cgD3D8DebugTrace)
    { 
        bool debugTrace = true;
    } else
    {
        bool otherError = true;
    }
}

//----------------------------------------------------------------------------
// d3d8Expanded()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
d3d8Expanded::d3d8Expanded() :
wire_(false),
vertexLighting_(true),
context_(NULL)
{
    cgD3D8EnableDebugTracing(true);
    cgSetErrorCallback(&cgErrorCallback);
}
d3d8Expanded::~d3d8Expanded()
{
}

//----------------------------------------------------------------------------
// ConfirmDevice()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
bool
d3d8Expanded::ConfirmDevice(const D3DCAPS8& caps, const D3DDEVICE_CREATION_PARAMETERS& d3dcp, const D3DPRESENT_PARAMETERS& d3dpp)
{
    if (d3dpp.EnableAutoDepthStencil == FALSE) return false;

	if (caps.PixelShaderVersion < D3DPS_VERSION(1,1))
	{
		return false;
	}
	if (d3dcp.BehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING || d3dcp.BehaviorFlags & D3DCREATE_MIXED_VERTEXPROCESSING)
	{
		if (caps.VertexShaderVersion < D3DVS_VERSION(1,1))
		{
			return false;
		}
	}
	return true;
}

//----------------------------------------------------------------------------
// OneTimeSceneInit()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::OneTimeSceneInit()
{
    return D3D_OK;
}

//----------------------------------------------------------------------------
// RebuildDevice()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::RebuildDevice()
{
    IDirect3DDevice8* pDev = D3DUtilContext::Instance()->GetD3DDevice();
    FAIL_RETURN(D3DUtilAppBasic::RebuildDevice());
    FAIL_RETURN(D3DXCreateTextureFromFile(pDev, STR_EARTH_MAP, &meshTexture_));
    CComPtr<IDirect3DTexture8> heightMap;
    // For simplicity, but this creates a managed texture with mipmaps that we don't use.
    FAIL_RETURN(D3DXCreateTextureFromFile(pDev, STR_EARTH_BUMP_MAP, &heightMap));
    D3DSURFACE_DESC desc;
    FAIL_RETURN(heightMap->GetLevelDesc(0, &desc));
    FAIL_RETURN(D3DXCreateTexture(pDev, desc.Width, desc.Height, 0, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &meshBump_));
    FAIL_RETURN(D3DXComputeNormalMap(meshBump_, heightMap, NULL, 0, D3DX_CHANNEL_RED, 10.f));
    
	// create the mesh (also get the bound-sphere)
	{
		CComPtr<ID3DXMesh> baseMesh;
		CComPtr<IDirect3DVertexBuffer8> vb;
        // Grump. Poor typedef'ing in D3D headers....
		FAIL_RETURN(D3DXLoadMeshFromX(const_cast<char*>(STR_CHAMELEON), D3DXMESH_SYSTEMMEM, pDev, NULL, NULL, NULL, &baseMesh));
		FAIL_RETURN(baseMesh->GetVertexBuffer(&vb));
		DWORD fvf = baseMesh->GetFVF();
		VOID* pVerts = NULL;
		FAIL_RETURN(vb->Lock(0, 0, reinterpret_cast<BYTE**>(&pVerts), D3DLOCK_READONLY));
		FAIL_RETURN(D3DXComputeBoundingSphere(pVerts, baseMesh->GetNumVertices(), fvf, &meshCenter_, &meshRadius_));
		FAIL_RETURN(vb->Unlock());
		FAIL_RETURN(baseMesh->CloneMeshFVF(D3DXMESH_MANAGED | D3DXMESH_WRITEONLY, finalFVF, pDev, &mesh_));
		FAIL_RETURN(D3DXComputeTangent(baseMesh, 0, mesh_, 1, D3DX_COMP_TANGENT_NONE, 0, NULL));
	}

    // Set the device and pick an appropriate profile
    context_ = cgCreateContext();
    cgD3D8SetDevice(pDev);
    VertexProfile = cgD3D8GetLatestVertexProfile();
    PixelProfile = cgD3D8GetLatestPixelProfile();

    // We reload the shaders every time since we pick a profile based on the device type
    FAIL_RETURN(_LoadVertexLightingPrograms());
    FAIL_RETURN(_LoadPixelLightingPrograms());

	return D3D_OK;
}

//----------------------------------------------------------------------------
// Reset()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::Reset()
{
    IDirect3DDevice8* pDev = D3DUtilContext::Instance()->GetD3DDevice();
    FAIL_RETURN(D3DUtilAppBasic::Reset());

	// create the point light position rep sphere.
	{
		CComPtr<ID3DXMesh> temp;
		FAIL_RETURN(D3DXCreateSphere(pDev, 0.5f, 10, 10, &temp, NULL));
		FAIL_RETURN(temp->CloneMeshFVF(D3DXMESH_WRITEONLY, D3DFVF_XYZ, pDev, &lightSphere_));
	}

    // Create the specular power lookup
    {
        CComPtr<IDirect3DSurface8> powSurf;
        FAIL_RETURN(pDev->CreateImageSurface(256, 1, D3DFMT_A8, &powSurf));
        D3DLOCKED_RECT r;
        FAIL_RETURN(powSurf->LockRect(&r, NULL, 0));
        for (int i=0;i<256;++i)
        {
            float NdotH = i * (1.f/255.f);
            static_cast<BYTE*>(r.pBits)[i] = static_cast<BYTE>(powf(NdotH, MatPower.x) * 255.f);
        }
        FAIL_RETURN(powSurf->UnlockRect());
        FAIL_RETURN(D3DXCreateTexture(pDev, 256, 1, 1, 0, D3DFMT_A8, D3DPOOL_DEFAULT, &meshSpecPow_));
        {
            CComPtr<IDirect3DSurface8> level0;
            FAIL_RETURN(meshSpecPow_->GetSurfaceLevel(0, &level0));
            FAIL_RETURN(D3DXLoadSurfaceFromSurface(level0, NULL, NULL, powSurf, NULL, NULL, D3DX_FILTER_NONE, 0x0));
        }
    }

    // set up common render states
	FAIL_RETURN(pDev->SetRenderState(D3DRS_SPECULARENABLE, TRUE));
	FAIL_RETURN(pDev->SetRenderState(D3DRS_FOGENABLE, FALSE));
	FAIL_RETURN(pDev->SetRenderState(D3DRS_LIGHTING, FALSE));

    // Have to re-bind this texture since we just recreated it.
    FAIL_RETURN(cgD3D8SetTexture(cgGetNamedParameter(pixLightParms_.pixprog_, "specPowMap"), meshSpecPow_));

	// set up the view the first time
	static bool once = false;
	if (!once)
	{
		ResetView();
		once = true;
	}

	return D3D_OK;
}

//----------------------------------------------------------------------------
// PrepareReset()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::PrepareReset()
{
    FAIL_RETURN(D3DUtilAppBasic::PrepareReset());
    lightSphere_ = NULL;
    meshSpecPow_ = NULL;
    // MeshSpecPow_ is a D3DPOOL_DEFAULT texture, so it needs to be
    // destroyed before the device can be reset.
    // But the expanded interface is holding a reference to it, also
    // so we must un-bind that parameter.
    FAIL_RETURN(cgD3D8SetTexture(cgGetNamedParameter(pixLightParms_.pixprog_, "specPowMap"), NULL));

	return D3D_OK;
}

//----------------------------------------------------------------------------
// InvalidateDevice()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::InvalidateDevice()
{
    IDirect3DDevice8* pDev = D3DUtilContext::Instance()->GetD3DDevice();
    FAIL_RETURN(D3DUtilAppBasic::InvalidateDevice());
	mesh_ = NULL;
	meshTexture_ = NULL;
	meshBump_ = NULL;

    // Since we are recreating all our programs based on the device type,
    // we destroy our programs every time we destroy the device
    cgDestroyContext(context_);
    // Tell the runtime to remove its reference to the D3D device.
    FAIL_RETURN(cgD3D8SetDevice(NULL));

	return D3D_OK;
}

//----------------------------------------------------------------------------
// UpdateScene()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::UpdateScene()
{
    FAIL_RETURN(D3DUtilAppBasic::UpdateScene());
	return D3D_OK;
}

//----------------------------------------------------------------------------
// Render()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::RenderScene()
{
    IDirect3DDevice8* pDev = D3DUtilContext::Instance()->GetD3DDevice();
	FAIL_RETURN(pDev->SetRenderState(D3DRS_FILLMODE, wire_ ? D3DFILL_WIREFRAME : D3DFILL_SOLID));

	D3DXMATRIX mv = GetCam().GetWorldToViewMatrix();
	D3DXMATRIX proj = GetCam().GetProjectionMatrix();

	// rotate the light
	D3DXVECTOR4 LightPosition = D3DXVECTOR4(0,1.f*meshRadius_,0.75f*meshRadius_,1);
	float rotAmt = fmodf(GetTotalFrameTime()/1000.f, 5.f)/ 5.f * 2.f * D3DX_PI;
	D3DXMATRIX rot;
	D3DXMatrixRotationAxis(&rot, &D3DXVECTOR3(0,1,0), rotAmt);
	D3DXVec4Transform(&LightPosition, &LightPosition, &rot);

	// bind the mvp matrix
	D3DXMATRIX mvp;
	D3DXMatrixMultiplyTranspose(&mvp, &mv, &proj);

    // just hard-code the material properties...
	D3DXVECTOR4 LightRange(20.f * meshRadius_,0,0,1);
	D3DXVECTOR4 LightAttenuation(1,1.f/255.f*meshRadius_,0,1);
	D3DXVECTOR4 CameraPosition(GetCam().GetPosition());

	// pulse the scale
	float scaleAmt = fmodf(GetTotalFrameTime()/1000.f, 3.f)/ 3.f * 2.f * D3DX_PI;
	scaleAmt = 0.5f + 0.5f*cosf(scaleAmt);
	D3DXVECTOR4 Scale(scaleAmt, 0.f, 0.f, 1.f);

	// begin the scene
    FAIL_RETURN(pDev->Clear(0, NULL, 
        D3DCLEAR_TARGET | 
        (D3DUtilContext::Instance()->IsDepth() ? D3DCLEAR_ZBUFFER : 0) | 
        (D3DUtilContext::Instance()->IsStencil() ? D3DCLEAR_STENCIL : 0), 
        0xFF000000, 1.f, 0x00));
	FAIL_RETURN(pDev->BeginScene());

    // bind the vb and ib
	{
		CComPtr<IDirect3DVertexBuffer8> vb;
		CComPtr<IDirect3DIndexBuffer8> ib;
		FAIL_RETURN(mesh_->GetVertexBuffer(&vb));
		FAIL_RETURN(mesh_->GetIndexBuffer(&ib));
		FAIL_RETURN(pDev->SetIndices(ib, 0));
		FAIL_RETURN(pDev->SetStreamSource(0, vb, D3DXGetFVFVertexSize(finalFVF)));
	}

	// set the shaders and bind the constants
    if (vertexLighting_)
    {
	    FAIL_RETURN(_SetupForVertexLighting(mvp, LightPosition, LightRange, LightAttenuation, CameraPosition, Scale));
    } else
    {
	    FAIL_RETURN(_SetupForPixelLighting(mvp, LightPosition, LightRange, LightAttenuation, CameraPosition, Scale));
    }

	// draw
	FAIL_RETURN(pDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, mesh_->GetNumVertices(), 0, mesh_->GetNumFaces()));

	// also draw the light position for clarity
	FAIL_RETURN(_ShowLightPos(mv, proj, LightPosition));

	// end the scene
    FAIL_RETURN(ShowStats());
	FAIL_RETURN(pDev->EndScene());
	FAIL_RETURN(pDev->Present(NULL, NULL, NULL, NULL));
	return D3D_OK;
}

//----------------------------------------------------------------------------
// _ShowLightPos()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
HRESULT
d3d8Expanded::_ShowLightPos(const D3DXMATRIX& modelview, const D3DXMATRIX& proj, const D3DXVECTOR4& lightPos)
{
    IDirect3DDevice8* pDev = D3DUtilContext::Instance()->GetD3DDevice();
	D3DXMATRIX trans;
	D3DXMatrixTranslation(&trans, lightPos.x, lightPos.y, lightPos.z);
	FAIL_RETURN(pDev->SetTransform(D3DTS_WORLD, &trans));
	FAIL_RETURN(pDev->SetTransform(D3DTS_VIEW, &modelview));
	FAIL_RETURN(pDev->SetTransform(D3DTS_PROJECTION, &proj));
	FAIL_RETURN(pDev->SetRenderState(D3DRS_TEXTUREFACTOR, 0xFFFFFF00));
	FAIL_RETURN(pDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2));
	FAIL_RETURN(pDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG2));
	FAIL_RETURN(pDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR));
	FAIL_RETURN(pDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TFACTOR));
	FAIL_RETURN(pDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE));
	FAIL_RETURN(pDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE));
	FAIL_RETURN(pDev->SetVertexShader(D3DFVF_XYZ));
	FAIL_RETURN(pDev->SetPixelShader(NULL));
	FAIL_RETURN(lightSphere_->DrawSubset(0));
	return D3D_OK;
}

//----------------------------------------------------------------------------
// WndProc()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
LRESULT
d3d8Expanded::WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& handled )
{
	// ***ALWAYS*** call the inherited WndProc first. This allows the navigation controls
	// and path recording to work properly. 
	// If that message was handled, it will set handled to true.
    LRESULT res = D3DUtilAppBasic::WndProc(hWnd, uMsg, wParam, lParam, handled);
    if (handled) return res;

	if (uMsg == WM_KEYDOWN)
	{
		if (wParam == 'B')
		{
			wire_ = !wire_;
		}
		if (wParam == VK_HOME)
		{
			ResetView();
		}
        handled = true;
    } else 
    if (uMsg == WM_COMMAND)
    {
        if (LOWORD(wParam) == IDM_LIGHTING_PIXEL)
        {
            vertexLighting_ = false;
        } else
        if (LOWORD(wParam) == IDM_LIGHTING_VERTEX)
        {
            vertexLighting_ = true;
        }
        handled = true;
    }
	return D3D_OK;
}

//----------------------------------------------------------------------------
// ResetView()
// ---------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
void
d3d8Expanded::ResetView()
{
	GetNav().SetWorldCenter(meshCenter_);
	GetNav().SetWorldRadius(meshRadius_);
	
	GetCam().LookAt(D3DXVECTOR3(0.f, 0.f, -2.f*GetNav().GetWorldRadius()), GetNav().GetWorldCenter(), GetNav().GetViewUp());
    GetCam().SetProjection(70.f, GetCam().GetAspect(), 0.5f, 20.f * GetNav().GetWorldRadius());
}


