//============================================================================
// File          : d3d9Minimal.hpp
// 
// Content       : Simple demo exercising the D3D9 Cg runtime minimal interfaces
// 
//============================================================================
// 
//============================================================================
#ifndef d3d9Minimal_included_
#define d3d9Minimal_included_
#pragma once


//----------------------------------------------------------------------------
//-- Includes ----------------------------------------------------------------
//----------------------------------------------------------------------------

#include "LibDX9/Util.hpp"
#include "LibDX9/D3DUtilAppBasic.hpp"
#include <atlbase.h>

#include "Cg/Cg.h"
#include "Cg/CgD3D9.h"

//----------------------------------------------------------------------------
//-- Forward Declarations ----------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Defines, Constants, Enumerated Types ------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Function Declarations ---------------------------------------------------
//----------------------------------------------------------------------------

void OutputErrors(const char* errs);


//----------------------------------------------------------------------------
//-- Typedefs, Structs, Classes ----------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// struct VertexLightingMembers
//----------------------------------------------------------------------------
// We have the vertex shader to do the lighting and a simple pixel shader to
// do the trivial base texture blending and specular addition.
//----------------------------------------------------------------------------
struct VertexLightingMembers
{
    VertexLightingMembers() { ZeroMemory(this, sizeof(*this)); }

    // vertex shader inputes
	CGprogram vertprog_;
    CComPtr<IDirect3DVertexShader9> vertprogHandle_;
    CComPtr<IDirect3DVertexDeclaration9> vertprogDecl_;
    UINT mvp_;
	UINT scale_;

	UINT matDiffuse_;
	UINT matSpecular_;
	UINT matAmbientEmissive_;
	UINT matSpecPower_;

	UINT lightPosition_;
	UINT lightRange_;
	UINT lightAttenuation_;

	UINT camPositonModelspace_;

    // pixel shader inputes
	CGprogram pixprog_;
    CComPtr<IDirect3DPixelShader9> pixprogHandle_;
	UINT baseTex_;
};

//----------------------------------------------------------------------------
// struct VertexLightingMembers
//----------------------------------------------------------------------------
// We have the vertex shader to do the bump mapping setup and a pixel shader to
// do the per-pixel lighting.
//----------------------------------------------------------------------------
struct PixelLightingMembers
{
    PixelLightingMembers() { ZeroMemory(this, sizeof(*this)); }

    // vertex shader inputes
	CGprogram vertprog_;
    CComPtr<IDirect3DVertexShader9> vertprogHandle_;
    CComPtr<IDirect3DVertexDeclaration9> vertprogDecl_;
    UINT mvp_;
	UINT scale_;

    UINT lightPosition_;
    UINT lightRange_;
    UINT lightAttenuation_;
	UINT camPositonModelspace_;

    // pixel shader inputes
	CGprogram pixprog_;
    CComPtr<IDirect3DPixelShader9> pixprogHandle_;
	UINT baseTex_;
	UINT bumpTex_;
	UINT specPowTex_;

    UINT matDiffuse_;
	UINT matSpecular_;
	UINT matAmbientEmissive_;
};

//----------------------------------------------------------------------------
// Class d3d9Minimal
//----------------------------------------------------------------------------
// 
//----------------------------------------------------------------------------
class d3d9Minimal : public D3DUtilAppBasic
{
public:
	d3d9Minimal();
	~d3d9Minimal();

	void ResetView();

    virtual HRESULT OneTimeSceneInit();
    virtual HRESULT RebuildDevice();
    virtual HRESULT Reset();
    virtual HRESULT PrepareReset();
    virtual HRESULT InvalidateDevice();
    virtual HRESULT UpdateScene();
    virtual HRESULT RenderScene();
    virtual LRESULT WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& handled );
    virtual bool ConfirmDevice(const D3DCAPS9& caps, const D3DDEVICE_CREATION_PARAMETERS& d3dcp, const D3DPRESENT_PARAMETERS& d3dpp);

private:
    HRESULT _LoadVertexLightingPrograms();
    HRESULT _LoadPixelLightingPrograms();
	HRESULT _SetupForVertexLighting(const D3DXMATRIX& modelviewproj, D3DXVECTOR4& lightPos, D3DXVECTOR4& lightRange, D3DXVECTOR4& lightAttenuation, D3DXVECTOR4& cameraPosition, D3DXVECTOR4& scale);
	HRESULT _SetupForPixelLighting(const D3DXMATRIX& modelviewproj, D3DXVECTOR4& lightPos, D3DXVECTOR4& lightRange, D3DXVECTOR4& lightAttenuation, D3DXVECTOR4& cameraPosition, D3DXVECTOR4& scale);
	HRESULT _ShowLightPos(const D3DXMATRIX& modelview, const D3DXMATRIX& proj, const D3DXVECTOR4& lightPos);

	CComPtr<ID3DXMesh> mesh_;
	CComPtr<ID3DXMesh> lightSphere_;
	CComPtr<IDirect3DTexture9> meshTexture_;
	CComPtr<IDirect3DTexture9> meshBump_;
	CComPtr<IDirect3DTexture9> meshSpecPow_;
	D3DXVECTOR3 meshCenter_;
	FLOAT meshRadius_;

	bool wire_;
    bool vertexLighting_;

	CGcontext context_;

    VertexLightingMembers vertLightParms_;
    PixelLightingMembers pixLightParms_;
};

const CGprofile PixelProfile = CG_PROFILE_PS_2_X;
const CGprofile VertexProfile = CG_PROFILE_VS_2_X;

//----------------------------------------------------------------------------
//-- External Variables ------------------------------------------------------
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//-- Function Definitions ----------------------------------------------------
//----------------------------------------------------------------------------


#endif
