#include "EventHandler.h"

#include "console/Console.h"

namespace GpuThesis {

EventHandler::EventHandler(Console::Console &console) : 
	SDL::EventHandler(true), isDone(false), isConsoleActive(false), console(&console) {}

bool EventHandler::done() const
{ 
	return isDone;
}

bool EventHandler::consoleActive() const
{
	return isConsoleActive;
}

void EventHandler::quit()
{ 
	isDone = true;
}
	
void EventHandler::handleChar(char key)
{
	if (key == '|') 
		isConsoleActive = !isConsoleActive;
	else if (key >= 32 && key <= 126 && isConsoleActive) // printable ASCII-characters
		console->handleChar(key);
	else if (!isConsoleActive)
		console->handleBinding(key);
}

void EventHandler::handleUpArrow()
{
	if (isConsoleActive)
		console->handleUpArrow();
}

void EventHandler::handleDownArrow()
{
	if (isConsoleActive)
		console->handleDownArrow();
}

void EventHandler::handleEnter()
{
	if (isConsoleActive)
		console->handleEnter();
}

void EventHandler::handleBackspace()
{
	if (isConsoleActive)
		console->handleBackspace();
}

void EventHandler::handleTab()
{
	if (isConsoleActive)
		console->handleTab();
}

std::string EventHandler::run(const std::vector<std::string> &/*parameters*/)
{
	isDone = true;
	return "";
}

}
