#ifndef GPUTHESIS_EVENTHANDLER_H
#define GPUTHESIS_EVENTHANDLER_H

#include "console/Action.h"

#include "sdl/Events.h"

namespace Console {

class Console;
}

namespace GpuThesis {

/**
 * Handles events for Engine.
 */
class EventHandler : public SDL::EventHandler, public Console::Action {
public:
	/**
	 * Initialize the Engine with the given Console instance.
	 */
	EventHandler(Console::Console &console);

	/**
	 * Check if the quit event has been sent.
	 * @return if the quit event has been sent.
	 */
	bool done() const;

	/**
	 * Check if the console is active or not.
	 * @return if the console is active.
	 */
	bool consoleActive() const;

private:
	void quit();	

	void handleChar(char key);
	void handleUpArrow();
	void handleDownArrow();
	void handleEnter();
	void handleBackspace();
	void handleTab();

	std::string run(const std::vector<std::string> &parameters);

	bool isDone;
	bool isConsoleActive;
	Console::Console *console;
};

}

#endif