#include "FluidFactory.h"

#include <memory>

namespace Fluid3dSolver {

FluidFactoryImpl::FluidFactoryImpl(bool slice) : slice(slice) {}

bool FluidFactoryImpl::isSliceSimulation() const
{
	return slice;
}

std::auto_ptr<Advection> FluidFactoryImpl::createAdvection(OpenGL::Flat3dTextureSet *textures,
	                                                       unsigned int velocityTexture, 
	                                                       unsigned int valueTexture,
	                                                       BoundaryType boundaryType)
{
	return std::auto_ptr<Advection>(
		new AdvectionImpl(textures,
		                  velocityTexture,
		                  valueTexture,
		                  boundaryType,
		                  slice));
}

std::auto_ptr<Advection> FluidFactoryImpl::createAdvection(OpenGL::Flat3dTextureSet *textures,
	                                                       unsigned int velocityTexture, 
	                                                       unsigned int valueTexture,
														   OpenGL::CgProgram *cgBoundary)
{
	return std::auto_ptr<Advection>(
		new AdvectionImpl(textures,
		                  velocityTexture,
		                  valueTexture,
		                  cgBoundary,
		                  slice));
}

std::auto_ptr<Diffusion> FluidFactoryImpl::createDiffusion(OpenGL::Flat3dTextureSet *textures,
	                                                       unsigned int valueTexture, 
	                                                       unsigned int tempTexture,
	                                                       BoundaryType boundaryType)
{
	return std::auto_ptr<Diffusion>(
		new DiffusionImpl(textures, 
		                  valueTexture,
		                  tempTexture,
		                  boundaryType,
		                  slice));
}

std::auto_ptr<Diffusion> FluidFactoryImpl::createDiffusion(OpenGL::Flat3dTextureSet *textures,
                                                           unsigned int valueTexture, 	                                         
                                                           BoundaryType boundaryType)
{
	return std::auto_ptr<Diffusion>(
		new DiffusionImpl(textures, 
		                  valueTexture,		                  
		                  boundaryType,
		                  slice));
}

std::auto_ptr<ForceAdder> FluidFactoryImpl::createForceAdder(OpenGL::Flat3dTextureSet *textures,
                                                             unsigned int valueTexture,
                                                             unsigned int forceTexture,
                                                             BoundaryType boundaryType)
{
	return std::auto_ptr<ForceAdder>(
		new ForceAdderImpl(textures,
		                   valueTexture,
		                   forceTexture,
		                   boundaryType,
		                   slice));
}

std::auto_ptr<ForceAdder> FluidFactoryImpl::createForceAdder(OpenGL::Flat3dTextureSet *textures,
                                                             unsigned int valueTexture,
                                                             unsigned int forceTexture,
															 OpenGL::CgProgram *cgBoundary)
{
	return std::auto_ptr<ForceAdder>(
		new ForceAdderImpl(textures,
		                   valueTexture,
		                   forceTexture,
		                   cgBoundary,
		                   slice));
}

std::auto_ptr<Projection> FluidFactoryImpl::createProjection(OpenGL::Flat3dTextureSet *textures,
                                                             unsigned int velocityTexture,
                                                             BoundaryType velocityBoundary)
{
	return std::auto_ptr<Projection>(
		new ProjectionImpl(textures,
		                   velocityTexture,
		                   velocityBoundary,
		                   slice));
}

std::auto_ptr<Projection> FluidFactoryImpl::createProjection(OpenGL::Flat3dTextureSet *textures,
                                                             unsigned int velocityTexture,
															 OpenGL::CgProgram *cgVelocityBoundary)
{
	return std::auto_ptr<Projection>(
		new ProjectionImpl(textures,
		                   velocityTexture,
		                   cgVelocityBoundary,
		                   slice));
}

std::auto_ptr<VorticityConfinement> 
	FluidFactoryImpl::createVorticityConfinement(OpenGL::Flat3dTextureSet *textures,
	                                             unsigned int velocityTexture, 
	                                             unsigned int velocityForceTexture,
	                                             unsigned int vorticityTexture)
{
	return std::auto_ptr<VorticityConfinement>(
		new VorticityConfinementImpl(textures,
		                             velocityTexture,
		                             velocityForceTexture,
		                             vorticityTexture,
		                             slice));
}

std::auto_ptr<VorticityConfinement> 
	FluidFactoryImpl::createVorticityConfinement(OpenGL::Flat3dTextureSet *textures,
	                                             unsigned int velocityTexture, 
	                                             unsigned int velocityForceTexture)
{
	return std::auto_ptr<VorticityConfinement>(
		new VorticityConfinementImpl(textures,
		                             velocityTexture,
		                             velocityForceTexture,		                             
		                             slice));
}

}

