#ifndef GPUTHESIS_LIGHT_H
#define GPUTHESIS_LIGHT_H

#include "EngineParameters.h"

#include "math/Vector3d.h"

#include "util/Color.h"

namespace GpuThesis {

/**
 * Represents a point light, which can be tied to the fire simulation.
 */
class Light {
public:
	Light();

	/**
	 * Initialize the light with the given parameters.
	 * @param light the light parameters.
	 */
	Light(EngineParameters::LightParams light);

	/**
	 * Get the light's position.
	 * @return the light's position.
	 */
	Math::Vector3d getPosition() const;

	/**
	 * Set the light's position.
	 * @param position the light's new position.
	 */
	void setPosition(const Math::Vector3d &position);	

	/**
	 * Get the light's color.
	 * @return the light's color.
	 */
	Util::Color getColor() const;

	/**
	 * Set the light's color.
	 * @param color the light's new color.
	 */
	void setColor(const Util::Color &color);

	/**
	 * Check if the light is dynamic.
	 * A dynamic light gets its color from a fire simulation.
	 * @return whether the light is dynamic.
	 */
	bool getDynamic() const;

	/** 
	 * Set whether the light is dynamic.
	 * @param dynamic whether the light is dynamic.
	 */
	void setDynamic(bool dynamic);

private:
	Math::Vector3d position;
	Util::Color color;
	bool dynamic;
};

}

#endif