#include "ModelObject.h"

#include "opengl/OpenGL.h"

namespace GpuThesis {


ModelObject::ModelObject(Geometry::Model *model, const EngineParameters::ModelObjectParams &params) :
	model(model),
	position(params.position),
	rotationAngle(params.rotationAngle),
	rotationAxis(params.rotationAxis),
	scale(params.scale),
	ignoreMeshes(params.ignoreMeshes),
	relative(params.relative),
	material(Material(params.material))
{
	for (unsigned int i = 0; i < model->meshes.size(); ++i) {
		if (params.faceNormalMeshes.count(i))
			model->meshes[i].useVertexNormals = false;
		else
			model->meshes[i].useVertexNormals = true;
	}
}

ModelObject::~ModelObject()
{	
}

void ModelObject::render(unsigned int meshNumber)
{
	model->render(ignoreMeshes);
}

void ModelObject::transform()
{
	if (relative)
		glTranslatef(position.x + anchor.x, position.y + anchor.y, position.z + anchor.z);
	else
		glTranslatef(position.x, position.y, position.z);

	glRotatef(rotationAngle, rotationAxis.x, rotationAxis.y, rotationAxis.z);

	glScalef(scale, scale, scale);
}

bool ModelObject::isLit()
{
	return true;
}

unsigned int ModelObject::getMeshCount()
{
	return 1;
}

const Material& ModelObject::getMaterial(unsigned int meshNumber)
{
	return material;
}	

void ModelObject::setAnchor(const Math::Vector3d &anchor)
{
	this->anchor = anchor;
}

}
