#ifndef GPUTHESIS_MODELOBJECT_H
#define GPUTHESIS_MODELOBJECT_H

#include <set>

#include "EngineParameters.h"
#include "Material.h"
#include "SceneObject.h"

#include "geometry/Model.h"
#include "math/Vector3d.h"

namespace GpuThesis {

/**
 * ModelObject is a specialization of SceneObject for Model.
 */
class ModelObject : public SceneObject {
public:
	/**
	 * Initialize the model object with the given Model and parameters.
	 * @param model the model to use when rendering.
	 * @param params the model object parameters.
	 */
	ModelObject(Geometry::Model *model, const EngineParameters::ModelObjectParams &params);
	~ModelObject();

	// From SceneObject
	/** 
	 * Render the given mesh.
	 * @param meshNumber the mesh to render.
	 */
	void render(unsigned int meshNumber);

	/**
	 * Transform the model object.
	 */
	void transform();

	/**
	 * Check if the model object should be lit.
	 * @return whethet the model object should be lit.
	 */
	bool isLit();

	/**
	 * Get the number of meshes in the model object.
	 * @return the number of meshes.
	 */
	unsigned int getMeshCount();

	/**
	 * Get the material for the given mesh.
	 * @param meshNumber the given mesh.
	 * @return the material.
	 */
	const Material& getMaterial(unsigned int meshNumber);

	/**
	 * Set the anchor to use for relative positions.
	 * @param anchor the anchor position.
	 */
	void setAnchor(const Math::Vector3d &anchor);

private:
	Geometry::Model *model;

	Math::Vector3d position;
	float rotationAngle;
	Math::Vector3d rotationAxis;
	float scale;
	bool relative;
	Math::Vector3d anchor;
	std::set<unsigned int> ignoreMeshes;
	Material material;
};

}

#endif