#ifndef GPUTHESIS_RUNNABLE_H
#define GPUTHESIS_RUNNABLE_H

namespace GpuThesis {

/**
 * Interface for any class that can be run over a single timestep.
 */
class Runnable {
public:
	virtual ~Runnable() = 0;

	/**
	 * Runs a single timestep.
	 * @param timestep The timestep to run.
	 */
	void run(float timestep);

	/**
	 * Reset the Runnable object.
	 */
	void reset();

private:
	/**
	 * Implemented by subclasses.
	 * @param timestep The timestep to run.
	 */
	virtual void doRun(float timestep) = 0;

	/**
	 * Implemented by subclasses.
	 */
	virtual void doReset() = 0;
};

}

#endif