#include "Runner.h"

#include "Runnable.h"

#include "util/Util.h"

namespace GpuThesis {


void Runner::add(GpuThesis::Runnable *runnable)
{ 
	runnables.push_back(runnable); 
}

void Runner::reset()
{
	for (RunnableVector::iterator iter = runnables.begin(); iter != runnables.end(); ++iter)
		(*iter)->reset();
}

std::string Runner::run(const std::vector<std::string> &parameters)
{
	if (parameters[0] == "reset") {
		reset();

		return "Reset all runnables.";
	}

	if (parameters.empty())
		return "Must specify timestep.";

	float timestep;

	if (!Util::lexical_cast(parameters[0], timestep)) 
		return "Parse error.";

	for (RunnableVector::iterator iter = runnables.begin(); iter != runnables.end(); ++iter)
		(*iter)->run(timestep);

	return "Ran timestep: " + parameters[0];
}

}
