#ifndef GPUTHESIS_RUNNER_H
#define GPUTHESIS_RUNNER_H

#include "console/Action.h"

namespace GpuThesis {

class Runnable;

/**
 * Specialization of Action which runs Runnables.
 */
class Runner : public Console::Action
{
public:
	/**
	 * Adds the given Runnable to the runner.
	 * @param runnable the Runnable to add.
	 */
	void add(GpuThesis::Runnable *runnable);

	/**
	 * Run the contained Runnables.
	 * The run parameters should be either a float specifying the timestep, or "reset" to reset all the Runnables.
	 * @param parameters the run parameters.
	 * @return "Parse error" or whether the runnables were reset or run with the given timestep.
	 */
	std::string run(const std::vector<std::string> &parameters);

	/**
	 * Reset all the Runnables.
	 */
	void reset();

private:
	typedef std::vector<GpuThesis::Runnable *> RunnableVector;
	RunnableVector runnables;
};

}

#endif