#ifndef GPUTHESIS_SCENEOBJECT_H
#define GPUTHESIS_SCENEOBJECT_H

#include "Material.h"

namespace GpuThesis {

/**
 * Virtual base class for scene objects.
 */
class SceneObject {
public:
	/**
	 * Render the given mesh of the scene object.
	 * A mesh in this context is a part of the scene object which has the same material.
	 * @param meshNumber the mesh to render.
	 */
	virtual void render(unsigned int meshNumber = 0) = 0;

	/**
	 * Transform the scene object.
	 * Transforms the scene object from object space to world space.
	 */
	virtual void transform() = 0;

	/**
	 * Check whether the scene object should be lit.
	 * @return whether the scene object should be lit.
	 */
	virtual bool isLit() = 0;

	/**
	 * Get the number of meshes in the scene object.
	 * @see render
	 * @return the mesh count.
	 */
	virtual unsigned int getMeshCount() = 0; 

	/**
	 * Get the Material of the given mesh.
	 * @param meshNumber the given mesh.
	 */
	virtual const Material& getMaterial(unsigned int meshNumber) = 0;

	virtual ~SceneObject() = 0;
};

}

#endif