#include "SimpleVolumeRenderer.h"

#include <memory>

#include "opengl/CgContext.h"
#include "opengl/CgProgram.h"
#include "opengl/Flat3dTextureSet.h"

#include "math/BoundingBox.h"

#include "util/Util.h"

using namespace Math;
using namespace Util;
using namespace OpenGL;

namespace GpuThesis {

SimpleVolumeRenderer::SimpleVolumeRenderer(Flat3dTextureSet *textures, 
										   unsigned int dataVolume)	:
	VolumeRenderer(BoundingBox(Vector3d(-0.5, 0, -0.5),
	                           Vector3d( 0.5, 1,  0.5))),
	computationWidth(textures->getComputationWidth()),
	computationHeight(textures->getComputationHeight()),
	computationDepth(textures->getComputationDepth())
{
	unsigned int width = 16;
	unsigned int height = 16;
	CgContext &context = CgContext::getInstance();

	svTextures = autoPtr(new IoTextureSet(width, height, 
	                                      context.getTexRectTarget(), 
	                                      context.getTexRectInternalFormat()));	

	unsigned int tiDataVolume = svTextures->importSecondaryTexture(textures, dataVolume, "Imported Data Volume");
	unsigned int tiOffsetLookup = svTextures->importSecondaryTexture(textures, textures->getOffsetLookup(), "Imported Offset Lookup");	
	
	cgDisplay = autoPtr(new CgProgram(context.getCgProgramPath() + "volume.cg", 
	                                          context.getFragmentShader(),
	                                          "displaySlice", 											  
	                                          svTextures.get()));

	cgDisplay->setTextureParameter("flat3dTexture", tiDataVolume);
	cgDisplay->setTextureParameter("offsetLookup", tiOffsetLookup);
}

void SimpleVolumeRenderer::doRender()
{	
	for (int d = computationDepth - 1; d >= 0; --d) {

		displaySlice(d);		
	}
}

OpenGL::IoTextureSet *SimpleVolumeRenderer::getTextureSet()
{
	return svTextures.get();
}

void SimpleVolumeRenderer::displaySlice(unsigned int sliceNumber) 
{
	float minX = -0.5f;
	float maxX = 0.5f;
	float minY = 0.0f;
	float maxY = 1.0f;
	float depth = 0.5f - static_cast<float>(sliceNumber) / 
		static_cast<float>(computationDepth);

	float minS = 0.0f;
	float maxS = static_cast<float>(computationWidth) - 1;
	float minT = 0.0f;
	float maxT = static_cast<float>(computationHeight) - 1;

	cgDisplay->setParameter("slice", static_cast<float>(sliceNumber));		

	glEnable(GL_BLEND);	
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);	

	cgDisplay->activate();

	glColor4f(1, 1, 1, 1);	

	glBegin(GL_QUADS);

	glTexCoord2f(minS, minT);	
	glVertex3f(minX, minY, depth);

	glTexCoord2f(maxS, minT);	
	glVertex3f(maxX, minY, depth);

	glTexCoord2f(maxS, maxT);	
	glVertex3f(maxX, maxY, depth);

	glTexCoord2f(minS, maxT);	
	glVertex3f(minX, maxY, depth);

	glEnd();

	cgDisplay->deactivate();

	glDisable(GL_BLEND);
}



}
