#ifndef GPUTHESIS_SIMPLEVOLUMERENDERER_H
#define GPUTHESIS_SIMPLEVOLUMERENDERER_H

#include <memory>

#include "VolumeRenderer.h"

namespace OpenGL {

class CgProgram;
class Flat3dTextureSet;
class IoTextureSet;
}

namespace GpuThesis {

/**
 * Specialization of VolumeRenderer which renders static slices.
 * Thus only looks good from certain directions, and is used for debugging and testing.
 * Note: not compatible with slice simulations.
 */
class SimpleVolumeRenderer : public VolumeRenderer {
public:
	/** 
	 * Initialize the SimpleVolumeRenderer.
	 * @param textures the texture set to read from.
	 * @param displayTexture the flat 3d texture to volume render.
	 */
	SimpleVolumeRenderer(OpenGL::Flat3dTextureSet *textures, unsigned int displayTexture);

	/**
	 * Get the internal texture set.
	 * @return the internal texture set.
	 */
	OpenGL::IoTextureSet *getTextureSet();

	/**
	 * SimpleVolumeRenderer does not use stepSize.
	 * @param stepSize step size.
	 */
	void setStepSize(float stepSize) {}

	/**
	 * SimpleVolumeRenderer does not use stepSize.
	 * @return 0.0f
	 */
	float getStepSize() const { return 0.0f; }

private:
	void doRender();
	void displaySlice(unsigned int sliceNumber);
	
	void updateBoundingBox() {}

	std::auto_ptr<OpenGL::IoTextureSet> svTextures;
	std::auto_ptr<OpenGL::CgProgram> cgDisplay;

	unsigned int computationWidth, computationHeight, computationDepth;
};

}

#endif