#include "VolumeRenderer.h"

#include "opengl/OpenGL.h"

using namespace Math;

namespace GpuThesis {

VolumeRenderer::VolumeRenderer(const BoundingBox &box) :
	box(box),
	drawBox(false)
{
}


VolumeRenderer::~VolumeRenderer()
{
}


void VolumeRenderer::render()
{
	if (drawBox)
		drawBoundingBox();

	doRender();
}


void VolumeRenderer::setBoundingBox(Math::BoundingBox box)
{
	this->box = box;

	updateBoundingBox();
}


Math::BoundingBox VolumeRenderer::getBoundingBox() const
{
	return box;
}


void VolumeRenderer::setDrawBoundingBox(bool drawBoundingBox)
{
	drawBox = drawBoundingBox;
}


bool VolumeRenderer::getDrawBoundingBox() const
{
	return drawBox;
}

#ifdef near
#undef near
#endif

#ifdef far
#undef far
#endif

void VolumeRenderer::drawBoundingBox() const
{
	glColor3f(1, 1, 1);

	glBegin(GL_LINE_LOOP);
	glVertex3f(box.getMin().x, box.getMin().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMin().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMin().y, box.getMax().z);
	glVertex3f(box.getMin().x, box.getMin().y, box.getMax().z);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3f(box.getMin().x, box.getMax().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMax().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMax().y, box.getMax().z);
	glVertex3f(box.getMin().x, box.getMax().y, box.getMax().z);
	glEnd();

	glBegin(GL_LINES);
	glVertex3f(box.getMin().x, box.getMin().y, box.getMin().z);
	glVertex3f(box.getMin().x, box.getMax().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMin().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMax().y, box.getMin().z);
	glVertex3f(box.getMax().x, box.getMin().y, box.getMax().z);
	glVertex3f(box.getMax().x, box.getMax().y, box.getMax().z);
	glVertex3f(box.getMin().x, box.getMin().y, box.getMax().z);
	glVertex3f(box.getMin().x, box.getMax().y, box.getMax().z);
	glEnd();
}

}