#ifndef GPUTHESIS_VOLUMERENDERER_H
#define GPUTHESIS_VOLUMERENDERER_H

#include "math/BoundingBox.h"

namespace OpenGL {
class IoTextureSet;
}

namespace GpuThesis {

/**
 * Base class for volume renderers.
 */
class VolumeRenderer {
public:
	/**
	 * Initialize the volume renderer with the given bounding box.
	 * @param box the bounding box.
	 */
	VolumeRenderer(const Math::BoundingBox &box);
	virtual ~VolumeRenderer();

	/**
	 * Perform the volume rendering.
	 */
	void render();

	/**
	 * Get the internal texture set (for debugging).
	 * @return the internal texture set.
	 */
	virtual OpenGL::IoTextureSet *getTextureSet() = 0;

	/**
	 * Set the bounding box.
	 * @param box the new bounding box.
	 */
	void setBoundingBox(Math::BoundingBox box);

	/**
	 * Get the bounding box.
	 * @return the bounding box.
	 */
	Math::BoundingBox getBoundingBox() const;

	/**
	 * Set whether to draw the bounding box.
	 * @param drawBoundingBox whether to draw the bounding box.
	 */
	void setDrawBoundingBox(bool drawBoundingBox);

	/**
	 * Check whether bounding box drawing is active.
	 * @return whether to draw the bounding box.
	 */
	bool getDrawBoundingBox() const;

	/**
	 * Set the step size of the volume renderer.
	 * Smaller step size gives more accurate results.
	 * @param stepSize the new step size.
	 */
	virtual void setStepSize(float stepSize) = 0;

	/**
	 * Get the step size of the volume renderer.
	 * @return the step size.
	 */
	virtual float getStepSize() const = 0;

	/**
	 * Perform per-frame computations.
	 */
	virtual void compute() {}

private:
	void drawBoundingBox() const;

	virtual void updateBoundingBox() = 0;
	virtual void doRender() = 0;

	Math::BoundingBox box;
	bool drawBox;
};

}

#endif
