#ifndef GPUTHESIS_TEST_FLUIDSLICETEST_H
#define GPUTHESIS_TEST_FLUIDSLICETEST_H

#include "GpuTest.h"

#include "Fluid3dSolver.h"

namespace OpenGL {
class Flat3dTextureSet;
}

namespace Fluid3dSolver {
class FluidFactory;
class Advection;
}

namespace GpuThesis {

class VolumetricExtrusion;

namespace Test {

class AdvectionSliceTest : public GpuTest {
public:
	AdvectionSliceTest();
	~AdvectionSliceTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOuput();

	unsigned int width, height, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> textures;
	std::auto_ptr<Fluid3dSolver::FluidFactory> factory;
	
	// Advection Interior
	std::auto_ptr<Fluid3dSolver::Advection> interior;
	unsigned int interiorVelocity, interiorValue;
	Fluid3dSolver::BoundaryType interiorBoundary;

	// Advection Boundary
	std::auto_ptr<Fluid3dSolver::Advection> boundary;
	unsigned int boundaryVelocity, boundaryValue;
	Fluid3dSolver::BoundaryType boundaryBoundary;

	// Velocity Slip Boundary
	std::auto_ptr<Fluid3dSolver::Advection> slip;
	unsigned int slipVelocity;
	Fluid3dSolver::BoundaryType slipBoundary;

	// Velocity No Slip Boundary
	std::auto_ptr<Fluid3dSolver::Advection> noSlip;
	unsigned int noSlipVelocity;
	Fluid3dSolver::BoundaryType noSlipBoundary;
};

class DiffusionSliceTest : public GpuTest {
public:
	DiffusionSliceTest();
	~DiffusionSliceTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOuput();

	unsigned int width, height, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> textures;
	std::auto_ptr<Fluid3dSolver::FluidFactory> factory;
	
	std::auto_ptr<Fluid3dSolver::Diffusion> diffusion;

	unsigned int valueTexture;
};

class DivergenceSliceTest : public GpuTest {
public:
	DivergenceSliceTest();
	~DivergenceSliceTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOutput();

	unsigned int width, height, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> textures;
	std::auto_ptr<Fluid3dSolver::FluidFactory> factory;
	
	std::auto_ptr<Fluid3dSolver::Projection> projection;

	unsigned int divergenceTexture;

	unsigned int velocityTexture;
};

class SubtractGradientSliceTest : public GpuTest {
public:
	SubtractGradientSliceTest();
	~SubtractGradientSliceTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOutput();

	unsigned int width, height, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> textures;
	std::auto_ptr<Fluid3dSolver::FluidFactory> factory;

	std::auto_ptr<OpenGL::CgProgram> cgSubtractGradient;
	std::auto_ptr<OpenGL::Flat3dTextureOperation> subtractGradientOp;
	unsigned int velocityTexture, pressureTexture;
};

class VorticityConfinementSliceTest : public GpuTest {
public:
	VorticityConfinementSliceTest();
	~VorticityConfinementSliceTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOutput();

	unsigned int width, height, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> textures;
	std::auto_ptr<Fluid3dSolver::FluidFactory> factory;

	std::auto_ptr<Fluid3dSolver::VorticityConfinement> vorticity;
	unsigned int velocityTexture, velocityForceTexture, vorticityTexture;

};

class VolumetricExtrusionTest : public GpuTest {
public:
	VolumetricExtrusionTest();
	~VolumetricExtrusionTest();

private:
	void doRun(float timestep);
	void doReset();

	void setInput();
	void setOutput();

	unsigned int sliceWidth, sliceHeight, numSlices;

	std::auto_ptr<OpenGL::Flat3dTextureSet> importTextures;
	unsigned int sliceTexture;

	std::auto_ptr<GpuThesis::VolumetricExtrusion> volumetricExtrusion;
};

}
}

#endif