#ifndef GEOMETRY_MESH_H
#define GEOMETRY_MESH_H

#include <vector>

#include "math/Definitions.h"
#include "math/Vector3d.h"

#include "util/Color.h"

namespace Geometry {

class Mesh {
public:
	Mesh();

	std::vector<Math::Vector3d> vertices;
	std::vector<Math::Vector3d> faceNormals;
	std::vector<Math::Vector3d> vertexNormals;
	std::vector<unsigned int> indices;

	Util::Color ambient, diffuse, specular;

	unsigned int vertexVBO;
	unsigned int normalVBO;
	unsigned int indexVBO;

	bool useVBO;

	bool useVertexNormals;

	void generateFaceNormals();
	void generateVertexNormals();

	void generateVBOs();
};

/**
 * Generate a sphere with the given radius and a triangle count of 
 * (4 * 4^detail).
 **/
void generateSphere(Mesh &target, Math::fp radius, unsigned int detail);

}

#endif
