#ifndef IMAGE_H
#define IMAGE_H

#include <string>

struct SDL_Surface;

/**
 * The Image namespace.
 */
namespace Image {

/**
 * The Image class represents a graphical image.
 */
class Image {
public:
	class LoadImageFailedException {};

	/**
	 * Constructs an Image from the specified image file.
	 * @param filename the image file.
	 */
	Image(const std::string &filename);
	~Image();

	/**
	 * Returns the width of the image.
	 * @return the width of the image.
	 */
	unsigned int getWidth() const;

	/**
	 * Returns the height of the image.
	 * @return the height of the image.
	 */
	unsigned int getHeight() const;

	/**
	 * Returns the byte per pixel in the image.
	 * @return the byte per pixel.
	 */
	unsigned int getBpp() const;

	/**
	 * Returns a pointer to the image data.
	 * TODO: how is the image data stored?
	 * @return a pointer to the image data.
	 */
	unsigned char *getData() const;

private:
	SDL_Surface *data;
};

}

#endif