#include "Matrix3x3.h" // class definition is found here

#include <cmath> // for sin, cos
#include <iostream>

namespace Math {

Matrix3x3::Matrix3x3() {
	setIdentity();
}

Matrix3x3::Matrix3x3(const Matrix3x3& m) {
	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			elements[i][j] = m.elements[i][j];
		}
	}
}

Matrix3x3::Matrix3x3(fp m[9]) {
	int index = 0;
	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
				elements[i][j] = m[index++];
		}
	}
}

Matrix3x3 Matrix3x3::operator*(const Matrix3x3& m) const {
	Matrix3x3 result;

	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			fp temp = 0;

			for(int k = 0; k < 3; ++k) {
				temp += elements[i][k] * m.elements[k][j];
			}

			result.elements[i][j] = temp;
		}
	}

	return result;
}

Vector3d Matrix3x3::operator*(const Vector3d& v) const {
	float result[3];

	float vf[3] = { v.x, v.y, v.z };

	for(int i = 0; i < 3; ++i) {
		fp temp = 0;

		for(int k = 0; k < 3; ++k) {
			temp += elements[i][k] * vf[k];
		}

		result[i] = temp;
	}

	return Vector3d(result[0], result[1], result[2]);
}

Matrix3x3 Matrix3x3::operator*(fp f) const {
	Matrix3x3 result;

	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			result.elements[i][j] = elements[i][j] * f;
		}
	}

	return result;
}

Matrix3x3 Matrix3x3::operator/(fp f) const {
	Matrix3x3 result;
	fp invf = 1.0f / f;

	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			result.elements[i][j] = elements[i][j] * invf;
		}
	}

	return result;
}

void Matrix3x3::setIdentity() {
	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			elements[i][j] = i==j ? (fp)1.0 : (fp)0.0;
		}
	}
}

fp Matrix3x3::getDeterminant() const {
	return elements[0][0] * (elements[1][1] * elements[2][2] - elements[1][2] * elements[2][1])
			- elements[0][1] * (elements[1][0] * elements[2][2] - elements[1][2] * elements[2][0])
			+ elements[0][2] * (elements[1][0] * elements[2][1] - elements[1][1] * elements[2][0]);
}

Matrix3x3 Matrix3x3::getInverse() const {
	Matrix3x3 inverse;

	inverse.elements[0][0] = elements[1][1] * elements[2][2] - elements[1][2] * elements[2][1];
	inverse.elements[0][1] = elements[0][2] * elements[2][1] - elements[0][1] * elements[2][2];
	inverse.elements[0][2] = elements[0][1] * elements[1][2] - elements[0][2] * elements[1][1];
	inverse.elements[1][0] = elements[1][2] * elements[2][0] - elements[1][0] * elements[2][2];
	inverse.elements[1][1] = elements[0][0] * elements[2][2] - elements[0][2] * elements[2][0];
	inverse.elements[1][2] = elements[0][2] * elements[1][0] - elements[0][0] * elements[1][2];
	inverse.elements[2][0] = elements[1][0] * elements[2][1] - elements[1][1] * elements[2][0];
	inverse.elements[2][1] = elements[0][1] * elements[2][0] - elements[0][0] * elements[2][1];
	inverse.elements[2][2] = elements[0][0] * elements[1][1] - elements[0][1] * elements[1][0];

	inverse = inverse / getDeterminant();

	return inverse;
}


std::ostream& operator<<(std::ostream& out, const Matrix3x3& m) {
	m.output(out);
	return out;
}

void Matrix3x3::output(std::ostream& out) const {
	for(int i = 0; i < 3; ++i) {
		for(int j = 0; j < 3; ++j) {
			out << elements[i][j] << " ";
		}
		out << std::endl;
	}
}
}
