#ifndef MATH_QUATERNION_H
#define MATH_QUATERNION_H

#include <cmath>
#include <iosfwd>

#include "Definitions.h"

namespace Math {

class Quaternion {
public:
	fp w, x, y, z;

	Quaternion() : w(1), x(0), y(0), z(0) {}
	Quaternion(fp w, fp x, fp y, fp z) : w(w), x(x), y(y), z(z) {}
};


inline Quaternion &operator+=(Quaternion &a, const Quaternion &b)
{
	a.w += b.w;
	a.x += b.x;
	a.y += b.y;
	a.z += b.z;
	
	return a;
}


inline Quaternion &operator*=(Quaternion &a, const Quaternion &b)
{
	fp w = a.w, x = a.x, y = a.y, z = a.z;

	a.w = w * b.w - x * b.x - y * b.y - z * b.z;
	a.x = w * b.x + x * b.w + y * b.z - z * b.y;
	a.y = w * b.y - x * b.z + y * b.w - z * b.x;
	a.z = w * b.z + x * b.y - y * b.x + z * b.w;
	
	return a;
}


inline const Quaternion operator*(const Quaternion &a, const Quaternion &b)
{
	Quaternion result;

	result.w = a.w * b.w - a.x * b.x - a.y * b.y - a.z * b.z;
	result.x = a.w * b.x + a.x * b.w + a.y * b.z - a.z * b.y;
	result.y = a.w * b.y - a.x * b.z + a.y * b.w - a.z * b.x;
	result.z = a.w * b.z + a.x * b.y - a.y * b.x + a.z * b.w;
	
	return result;
}


inline const Quaternion operator/(const Quaternion &q, fp f)
{
	fp inv = 1 / f;

	return Quaternion(q.w * inv, q.x * inv, q.y * inv, q.z * inv);
}


inline fp norm(const Quaternion &q)
{
	return q.w * q.w + q.x * q.x + q.y * q.y + q.z * q.z;
}


inline fp mod(const Quaternion &q)
{
	return std::sqrt(norm(q));
}


inline const Quaternion conjugate(const Quaternion &q)
{
	return Quaternion(q.w, -q.x, -q.y, -q.z);
}


inline const Quaternion inverse(const Quaternion &q)
{
	return conjugate(q) / norm(q);
}


const float *getMatrix(const Quaternion &q);

}

#endif