#include "opengl/Camera.h"

#include <cmath>
#include <iostream>

#include "GL/glew.h"

#include "math/Quaternion.h"

namespace OpenGL {

Camera::Camera(Math::Vector3d pos) :
	pitchAngle(0),
	yawAngle(0),
	pos(pos)
{
}

Math::Vector3d Camera::getPosition() const
{
	return pos;
}

Math::Vector3d Camera::getViewDirection() const
{
	const float* matrix = getMatrix();
	return Math::Vector3d(matrix[2], matrix[6], matrix[10]);
}

void Camera::move(float dist) 
{
	const float* matrix = getMatrix();
	pos.x += matrix[2] * dist;
	pos.y += matrix[6] * dist;
	pos.z -= matrix[10] * dist;
}

void Camera::strafe(float dist)
{
	const float* matrix = getMatrix();
	pos.x += matrix[0] * dist;
	pos.y += matrix[4] * dist;
	pos.z -= matrix[8] * dist;
}

void Camera::up(float dist)
{
	const float* matrix = getMatrix();
	pos.x += matrix[1] * dist;
	pos.y += matrix[5] * dist;
	pos.z -= matrix[9] * dist;
}

void Camera::pitch(float angle, bool limitRotation)
{
	pitchAngle += angle;

	if (limitRotation) {
		float limit = Math::pi / 2 - Math::pi / 36;

		if (pitchAngle < -limit)
			pitchAngle = -limit;

		if (pitchAngle > limit)
			pitchAngle = limit;
	}
}

void Camera::yaw(float angle)
{
	yawAngle += angle;
}

void Camera::transform() const
{
	const float* matrix = getMatrix();

	gluLookAt(pos.x, pos.y, pos.z, pos.x + matrix[2], pos.y + matrix[6], pos.z - matrix[10], 
		                             matrix[1], matrix[5], -matrix[9]);
}

const float* Camera::getMatrix() const
{
	Math::Quaternion rotation(1, 0, 0, 0);

	float cYaw = cos(yawAngle / 2.0f);
	float sYaw = sin(yawAngle / 2.0f);

	float cPitch = cos(pitchAngle / 2.0f);
	float sPitch = sin(pitchAngle / 2.0f);

	rotation *= Math::Quaternion(cYaw, 0, sYaw, 0);
	rotation *= Math::Quaternion(cPitch, sPitch, 0, 0);

	return Math::getMatrix(rotation);
}

}

