#include "CgContext.h"

#include <cassert>

#include <Cg/cg.h>
#include <Cg/cgGL.h>

namespace OpenGL {

CgContext::CgContext() : context(cgCreateContext())
{
}


CgContext::~CgContext()
{
	cgDestroyContext(context);
}


bool CgContext::good() const
{
	return context != 0;
}


CgContext &CgContext::getInstance()
{
	static CgContext context;

	return context;
}


void CgContext::enableProfile(CGprofile profile) const
{
	assert(good());

	cgGLEnableProfile(profile);
}


void CgContext::disableProfile(CGprofile profile) const
{
	assert(good());

	cgGLDisableProfile(profile);
}


bool CgContext::hasError() const
{
	CGerror error = cgGetError();

	lastError = cgGetErrorString(error);

	return error != CG_NO_ERROR;
}


std::string CgContext::getError() const
{
	return lastError;
}


CGcontext CgContext::getContext() const
{
	return context;
}


CGprofile CgContext::getFragmentShader() const
{
	if (cgGLIsProfileSupported(CG_PROFILE_FP40))
		return CG_PROFILE_FP40;
	else
		return CG_PROFILE_UNKNOWN;
}


CGprofile CgContext::getVertexShader() const
{
	if (cgGLIsProfileSupported(CG_PROFILE_VP40))
		return CG_PROFILE_VP40;
	else
		return CG_PROFILE_UNKNOWN;
}


std::string CgContext::getCgProgramPath() const
{
	return "src/cg/";
}


GLenum CgContext::getTexRectTarget() const
{
	return GL_TEXTURE_RECTANGLE_NV;
}


GLint CgContext::getTexRectInternalFormat() const
{
	return GL_FLOAT_RGBA32_NV;
}


GLenum CgContext::getTex2dTarget() const
{
	return GL_TEXTURE_2D;
}


GLint CgContext::getTex2dInternalFormat() const
{
	return GL_RGBA_FLOAT32_ATI;
}

}
