#include "CgFactory.h"

#include "CgContext.h"
#include "CgProgram.h"
#include "Flat3dTextureOperation.h"
#include "Texture2dOperation.h"
#include "Flat3dTextureSet.h"

namespace OpenGL {

std::auto_ptr<Flat3dTextureOperation>
createFlat3dTextureOperation(const Flat3dTextureSet *textures,
                             CgProgram *interior,
                             Flat3dTextureOperation::OperationType operation,
                             bool normalizedCoordinates)
{
	return std::auto_ptr<Flat3dTextureOperation>(
		new Flat3dTextureOperation(textures->getComputationWidth(),
		                           textures->getComputationHeight(),
		                           textures->getLayoutWidth(),
		                           textures->getLayoutHeight(),
		                           textures->getComputationDepth(),
		                           interior,
		                           operation,
		                           normalizedCoordinates));
}

std::auto_ptr<Flat3dTextureOperation>
createFlat3dTextureOperation(const Flat3dTextureSet *textures,
                             CgProgram *interior,
                             CgProgram *boundary,
                             Flat3dTextureOperation::OperationType operation,
                             bool normalizedCoordinates)
{
	return std::auto_ptr<Flat3dTextureOperation>(
		new Flat3dTextureOperation(textures->getComputationWidth(),
		                           textures->getComputationHeight(),
		                           textures->getLayoutWidth(),
		                           textures->getLayoutHeight(),
		                           textures->getComputationDepth(),
		                           interior,
		                           boundary,
		                           operation,
		                           normalizedCoordinates));
}

std::auto_ptr<CgProgram>
createFragmentProgram(const std::string &program,
                      const std::string &entry,
                      IoTextureSet *textures)
{
	const CgContext &context = OpenGL::CgContext::getInstance();

	return std::auto_ptr<OpenGL::CgProgram>(
		new CgProgram(context.getCgProgramPath() + program, 
		              context.getFragmentShader(), 
		              entry, 
		              textures));
}

}
