#ifndef OPENGL_CGFACTORY_H
#define OPENGL_CGFACTORY_H

#include <memory>

#include "Flat3dTextureOperation.h"


namespace OpenGL {

class CgProgram;
class Texture2dOperation;

class Flat3dTextureSet;

/**
 * Creates a Flat3dTextureOperation instance.
 * @param textures the texture set used by the Flat3dTextureOperation instance.
 * @param interior the CgProgram used for interior computations.
 * @param operation the operation type of the Flat3dTextureOperation instance.
 * @param normalizedCoordinates whether normalized texture coordinates is used.
 * @return an auto pointer to the created Flat3dTextureOperation instance.
 */
std::auto_ptr<Flat3dTextureOperation>
createFlat3dTextureOperation(const Flat3dTextureSet *textures,
                             CgProgram *interior,
                             Flat3dTextureOperation::OperationType operation,
                             bool normalizedCoordinates = false);

/**
 * Creates a Flat3dTextureOperation instance.
 * @param textures the texture set used by the Flat3dTextureOperation instance.
 * @param interior the CgProgram used for interior computations.
 * @param boundary the CgProgram used for boundary computations.
 * @param operation the operation type of the Flat3dTextureOperation instance.
 * @param normalizedCoordinates whether normalized texture coordinates is used.
 * @return an auto pointer to the created Flat3dTextureOperation instance.
 */
std::auto_ptr<Flat3dTextureOperation>
createFlat3dTextureOperation(const Flat3dTextureSet *textures,
                             CgProgram *interior,
                             CgProgram *boundary,
                             Flat3dTextureOperation::OperationType operation,
                             bool normalizedCoordinates = false);

/**
 * Creates a fragment CgProgram instance based on a given function in the 
 * given Cg file. The Cg file is assumed located relative to the Cg program path
 * retrieved by the CgContext instance.
 * @param program the name of the Cg file.
 * @param entry the name of the function in the Cg file.
 * @param textures the texture set used by the CgProgram.
 * @return an auto pointer to the created CgProgram instance.
 */
std::auto_ptr<CgProgram>
createFragmentProgram(const std::string &program,
                      const std::string &entry,
                      IoTextureSet *textures);

}

#endif