#include "CgFragmentProgram.h"

#include <iostream>

#include <GL/glew.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>

#include "CgContext.h"

namespace OpenGL {

/***** Public methods *****/

CgFragmentProgram::CgFragmentProgram(const std::string& programFile, CGprofile profile,
                     const std::string& entryPoint, IoTextureSet* ioTextureSet,
					 CgFragmentProgram::SlabOperationType slabOpType) :
	CgProgram(programFile, profile, entryPoint, ioTextureSet),		
	minX(0), minY(0), maxX(1), maxY(1),
	minS(0), minT(0), maxS(1), maxT(1),
	texResWidth(1), texResHeight(1),
	slabOpType(slabOpType)
{
	setBoundaryParameters();
}

void CgFragmentProgram::compute()
{
	setState();

	if(slabOpType == RECTANGLE)
		drawQuad();
	else
		drawBoundary();

	resetState();
}


void CgFragmentProgram::setQuadDimensions(float minX, float minY, float maxX, float maxY)
{
	this->minX = minX;
	this->minY = minY;
	this->maxX = maxX;
	this->maxY = maxY;

	setBoundaryParameters();
}


void CgFragmentProgram::setTextureCoordinates(float minS, float minT, float maxS, float maxT)
{
	this->minS = minS;
	this->minT = minT;
	this->maxS = maxS;
	this->maxT = maxT;

	setBoundaryParameters();
}


void CgFragmentProgram::setTextureResolution(float width, float height)
{
	texResWidth = width;
	texResHeight = height;
}

/***** Private methods *****/
void CgFragmentProgram::drawQuad()
{
	glBegin(GL_QUADS);

	glTexCoord2f(minS, minT);
	glVertex2f(minX, minY);

	glTexCoord2f(maxS, minT);
	glVertex2f(maxX, minY);

	glTexCoord2f(maxS, maxT);
	glVertex2f(maxX, maxY);

	glTexCoord2f(minS, maxT);
	glVertex2f(minX, maxY);

	glEnd();
}

void CgFragmentProgram::drawBoundary()
{

    glBegin(GL_LINES);

	// left boundary
	glMultiTexCoord2f(GL_TEXTURE1_ARB, texelWidth, 0); // offset amount
	glTexCoord2f(minS, minT);
	glVertex2f(minX, minY);
	glTexCoord2f(minS, maxT);
	glVertex2f(minX, maxY);

	// right boundary
	glMultiTexCoord2f(GL_TEXTURE1_ARB, -texelWidth, 0); // offset amount
	glTexCoord2f(maxS - texelWidth, minT);
	glVertex2f(maxX - pixelWidth, minY);
	glTexCoord2f(maxS - texelWidth, maxT);
	glVertex2f(maxX - pixelWidth, maxY);

	// bottom boundary
	glMultiTexCoord2f(GL_TEXTURE1_ARB, 0, texelHeight); // offset amount
	glTexCoord2f(minS, minT);
	glVertex2f(minX, minY + 0.5f * pixelHeight);
	glTexCoord2f(maxS, minT);
	glVertex2f(maxX, minY + 0.5f * pixelHeight);

	// top boundary
	glMultiTexCoord2f(GL_TEXTURE1_ARB, 0, -texelHeight); // offset amount
	glTexCoord2f(minS, maxT - texelHeight);
	glVertex2f(minX, maxY - 0.5f * pixelHeight);
	glTexCoord2f(maxS, maxT - texelHeight);
	glVertex2f(maxX, maxY - 0.5f * pixelHeight);

    glEnd();
}


bool CgFragmentProgram::isInputTexture(unsigned int texture) const
{
	for (TextureMap::const_iterator i = textureMap.begin(), end = textureMap.end(); i != end; ++i) {
		if (i->second == texture)
			return true;
	}

	return false;
}


void CgFragmentProgram::setBoundaryParameters()
{

    pixelWidth  = (maxX - minX) / (maxS - minS);
    pixelHeight = (maxY - minY) / (maxT - minT);

    texelWidth  = (maxS - minS) / texResWidth;
    texelHeight = (maxT - minT) / texResHeight;
}

}