#ifndef OPENGL_CGFRAGMENTPROGRAM_H
#define OPENGL_CGFRAGMENTPROGRAM_H

#include <GL/glew.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>

#include "opengl/IoTextureSet.h"
#include "opengl/CgProgram.h"

#include <string>
#include <memory>
#include <vector>
#include <map>

namespace OpenGL {

class CgFragmentProgram : public CgProgram {

public:

	enum SlabOperationType {
		BOUNDARY,
		RECTANGLE
	};

	CgFragmentProgram(const std::string& programFile, CGprofile profile,
	                  const std::string& entryPoint,
	                  IoTextureSet* ioTextureSet,
	                  SlabOperationType slabOpType = RECTANGLE);

	void compute();

	void setQuadDimensions(float minX, float minY, float maxX, float maxY);
	void setTextureCoordinates(float minS, float minT, float maxS, float maxT);
	void setTextureResolution(float width, float height);	

	bool isInputTexture(unsigned int texture) const;

private:	
	// execution parameters
	float minX, minY, maxX, maxY;
	float minS, minT, maxS, maxT;

	// extra boundary parameters
	float texResWidth, texResHeight;
	float texelWidth, texelHeight, pixelWidth, pixelHeight;

	// slab operation type
	SlabOperationType slabOpType;

	void drawQuad();
	void drawBoundary();
	void setBoundaryParameters();
};

}


#endif