#ifndef OPENGL_FPSCOUNTER_H
#define OPENGL_FPSCOUNTER_H

#include <vector>

namespace OpenGL {

/**
 * The FpsCounter class monitors the frame rate and renders
 * it to the framebuffer. 
 */
class FpsCounter {
public:
	/**
	 * Constructor.
	 * @param numTimeSteps the number of timesteps saved in order to compute the FPS.
	 */
	FpsCounter(unsigned int numTimeSteps = 50);

	/**
	 * Updates the FPS value based on the last timestep.
	 * @param timestep the timestep since the last frame.
	 */
	void update(float timestep);

	/**
	 * Renders the FPS value to the specified framebuffer position.
	 * @param x the x-coordinate of the framebuffer position.
	 * @param y the y-coordinate of the framebuffer position.
	 */
	void render(unsigned int x, unsigned int y) const;

private:
	float calculateFps() const;
	float calculateVariance() const;

	unsigned int numTimeSteps;

	std::vector<float> timesteps;

	int currentIndex;
};

}

#endif
