#include <algorithm>
#include <functional>

#include "PrimaryTextureContainer.h"

#include "IoTextureSet.h"

namespace OpenGL {

PrimaryTextureContainer::
PrimaryTextureContainer(IoTextureSet *textureSet,
                        unsigned int viewportX, unsigned int viewportY,
                        unsigned int viewportWidth, unsigned int viewportHeight) :
	TextureContainer(textureSet, viewportX, viewportY, viewportWidth, viewportHeight),
	textureSet(textureSet)
{
	initialize();
}


PrimaryTextureContainer::PrimaryTextureContainer(IoTextureSet *textureSet) :
	TextureContainer(textureSet),
	textureSet(textureSet)
{
	initialize();
}


GLenum PrimaryTextureContainer::getActiveTarget() const
{
	return textureSet->getPrimaryTarget();
}


unsigned int PrimaryTextureContainer::getActiveWidth() const
{
	return textureSet->getPrimaryWidth();
}


unsigned int PrimaryTextureContainer::getActiveHeight() const
{
	return textureSet->getPrimaryHeight();
}


GLenum PrimaryTextureContainer::getTarget(unsigned int /*texture*/) const
{
	return textureSet->getPrimaryTarget();
}


unsigned int PrimaryTextureContainer::getWidth(unsigned int /*texture*/) const
{
	return textureSet->getPrimaryWidth();
}


unsigned int PrimaryTextureContainer::getHeight(unsigned int /*texture*/) const
{
	return textureSet->getPrimaryHeight();
}


void PrimaryTextureContainer::handleTextureActivation(unsigned int /*texture*/)
{
}


void PrimaryTextureContainer::initialize()
{
	std::vector<unsigned int> primaryTextures = textureSet->getPrimaryTextures();

	std::for_each(primaryTextures.begin(), primaryTextures.end(), 
		std::bind1st(
			std::mem_fun(&PrimaryTextureContainer::insertTexture), 
			this));
}

}