#ifndef OPENGL_PRIMARYTEXTURECONTAINER_H
#define OPENGL_PRIMARYTEXTURECONTAINER_H

#include "OpenGL.h"
#include "TextureContainer.h"

namespace OpenGL {

class IoTextureSet;

/**
 * Contains a set of primary textures from an IoTextureSet and displays them on the
 * screen.
 */
class PrimaryTextureContainer : public TextureContainer {
public:
	/**
	 * Initializes the PrimaryTextureContainer based on the given IoTextureSet and 
	 * viewport dimensions.
	 * @param textureSet the given IoTextureSet.
	 * @param viewportX the left position of the viewport.
	 * @param viewportY the top position of the viewport.
	 * @param viewportWidth the width of the viewport.
	 * @param viewportHeight the height of the viewport.
	 */
	PrimaryTextureContainer(IoTextureSet *textureSet,
	                        unsigned int viewportX, unsigned int viewportY,
	                        unsigned int viewportWidth, unsigned int viewportHeight);

	/**
	 * Initializes the PrimaryTextureContainer based on the given IoTextureSet,
	 * and the current OpenGL viewport coordinates.
	 * @param textureSet the given IoTextureSet.
	 */
	PrimaryTextureContainer(IoTextureSet *textureSet);

private:
	GLenum getActiveTarget() const;
	unsigned int getActiveWidth() const;
	unsigned int getActiveHeight() const;

	GLenum getTarget(unsigned int textureNumber) const;
	unsigned int getWidth(unsigned int textureNumber) const;
	unsigned int getHeight(unsigned int textureNumber) const;

	void handleTextureActivation(unsigned int textureNumber);

	void initialize();

	IoTextureSet *textureSet;
};

}

#endif