#include "Rendering.h"

#include "OpenGL.h"

#include "geometry/Mesh.h"

namespace OpenGL {
  
void displayMesh(const Geometry::Mesh &mesh)
{
	if (mesh.useVBO) {
		glColor3f(mesh.diffuse.getRed(), mesh.diffuse.getGreen(), mesh.diffuse.getBlue());

		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);

		glBindBufferARB(GL_ARRAY_BUFFER_ARB, mesh.vertexVBO);
		glVertexPointer(3, GL_FLOAT, 0, 0);
		glBindBufferARB(GL_ARRAY_BUFFER_ARB, mesh.normalVBO);
		glNormalPointer(GL_FLOAT, 0, 0);

		glDrawArrays(GL_TRIANGLES, 0, static_cast<unsigned int>(mesh.vertices.size()) / 3);

		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);
	} else {
		glColor3f(mesh.diffuse.getRed(), mesh.diffuse.getGreen(), mesh.diffuse.getBlue());

		glBegin(GL_TRIANGLES);

		for (unsigned int i = 0; i < mesh.indices.size(); i += 3) {
			if (!mesh.useVertexNormals) {
					const Math::Vector3d &normal = mesh.faceNormals[i / 3];
					glNormal3f(normal.x, normal.y, normal.z);
			}
			for (unsigned int j = 0; j < 3; ++j) {
				const Math::Vector3d &vertex = mesh.vertices[mesh.indices[i+j]];

				if (mesh.useVertexNormals) {
					const Math::Vector3d &normal = mesh.vertexNormals[mesh.indices[i+j]];
					glNormal3f(normal.x, normal.y, normal.z);
				}
				glVertex3d(vertex.x, vertex.y, vertex.z);
			}
		}

		glEnd();
	}
}



void displayString(void *font, 
                   const std::string &str)
{
	for (unsigned int j = 0; j < str.size(); ++j)
		glutBitmapCharacter(font, str[j]);
}



void displayString(void *font,
                   const std::string &str, 
                   unsigned int maxWidth)
{
	unsigned int currentWidth = 0;

	for (unsigned int j = 0; j < str.size(); ++j) {
		currentWidth += glutBitmapWidth(font, str[j]);

		if (currentWidth > maxWidth)
			break;

		glutBitmapCharacter(font, str[j]);
	}
}

}
