#include <algorithm>
#include <cassert>
#include <functional>

#include "SecondaryTextureContainer.h"

#include "IoTextureSet.h"

namespace OpenGL {

SecondaryTextureContainer::
SecondaryTextureContainer(IoTextureSet *textureSet,
                          unsigned int viewportX, unsigned int viewportY,
						  unsigned int viewportWidth, unsigned int viewportHeight) :
	TextureContainer(textureSet, viewportX, viewportY, viewportWidth, viewportHeight),
	activeTexture(0)
{
	initialize(textureSet);
}


SecondaryTextureContainer::
SecondaryTextureContainer(IoTextureSet *textureSet) :
	TextureContainer(textureSet),
	activeTexture(0)
{
	initialize(textureSet);
}


GLenum SecondaryTextureContainer::getActiveTarget() const
{
	assert(isActive(activeTexture));

	return secondaryTextures[activeTexture].target;
}


unsigned int SecondaryTextureContainer::getActiveWidth() const
{
	assert(isActive(activeTexture));

	return secondaryTextures[activeTexture].width;
}


unsigned int SecondaryTextureContainer::getActiveHeight() const
{
	assert(isActive(activeTexture));

	return secondaryTextures[activeTexture].height;
}


GLenum SecondaryTextureContainer::getTarget(unsigned int texture) const
{
	return secondaryTextures[texture].target;
}


unsigned int SecondaryTextureContainer::getWidth(unsigned int texture) const
{
	return secondaryTextures[texture].width;
}


unsigned int SecondaryTextureContainer::getHeight(unsigned int texture) const
{
	return secondaryTextures[texture].height;
}


void SecondaryTextureContainer::handleTextureActivation(unsigned int texture)
{
	activeTexture = texture;

	for (unsigned int i = 0; i < getTextureCount(); ++i) {
		if (i != activeTexture)
			deactivateTexture(i);
	}
}


void SecondaryTextureContainer::initialize(IoTextureSet *textureSet)
{
	std::vector<unsigned int> secondaryTextures = textureSet->getSecondaryTextures();

	for (std::vector<unsigned int>::const_iterator iter = secondaryTextures.begin();
		 iter != secondaryTextures.end(); ++iter)
	{
		unsigned int textureId = *iter;

		this->secondaryTextures.push_back(
			TextureData(
				textureSet->getWidth(textureId),
				textureSet->getHeight(textureId),
				textureSet->getTarget(textureId)));
			       
	}

	std::for_each(secondaryTextures.begin(), secondaryTextures.end(), 
		std::bind1st(
			std::mem_fun(&SecondaryTextureContainer::insertTexture), 
			this));
}

}
