#ifndef OPENGL_SECONDARYTEXTURECONTAINER_H
#define OPENGL_SECONDARYTEXTURECONTAINER_H

#include <vector>

#include "OpenGL.h"
#include "TextureContainer.h"

namespace OpenGL {

class IoTextureSet;

/**
 * Contains a set of secondary textures from an IoTextureSet and displays one of them on the
 * screen.
 */
class SecondaryTextureContainer : public TextureContainer {
public:
	/**
	 * Initializes the SecondaryTextureContainer based on the given IoTextureSet and 
	 * viewport dimensions.
	 * @param textureSet the given IoTextureSet.
	 * @param viewportX the left position of the viewport.
	 * @param viewportY the top position of the viewport.
	 * @param viewportWidth the width of the viewport.
	 * @param viewportHeight the height of the viewport.
	 */
	SecondaryTextureContainer(IoTextureSet *textureSet,
	                        unsigned int viewportX, unsigned int viewportY,
	                        unsigned int viewportWidth, unsigned int viewportHeight);

	/**
	 * Initializes the SecondaryTextureContainer based on the given IoTextureSet,
	 * and the current OpenGL viewport coordinates.
	 * @param textureSet the given IoTextureSet.
	 */
	SecondaryTextureContainer(IoTextureSet *textureSet);

private:
	GLenum getActiveTarget() const;
	unsigned int getActiveWidth() const;
	unsigned int getActiveHeight() const;

	GLenum getTarget(unsigned int textureNumber) const;
	unsigned int getWidth(unsigned int textureNumber) const;
	unsigned int getHeight(unsigned int textureNumber) const;

	void handleTextureActivation(unsigned int textureNumber);

	void initialize(IoTextureSet *textureSet);

	struct TextureData {
		unsigned int width, height;
		GLenum target;

		TextureData(unsigned int width, unsigned int height, GLenum target) :
			width(width), height(height), target(target) {}
	};

	std::vector<TextureData> secondaryTextures;

	unsigned int activeTexture;
};

}

#endif