#ifndef OPENGL_TEXTURE2DOPERATION_H
#define OPENGL_TEXTURE2DOPERATION_H

#include "CgOperation.h"

namespace OpenGL {

class CgProgram;

/**
 * Texture2dOperation is a specialization of CgOperation for standard 2d texture operations
 * which operate on a single rectangular texture. It draws a quad to compute the interior
 * CgProgram. If the boundary is activated, it draws a frame surrounding the quad to compute
 * the boundary CgProgram.
 */
class Texture2dOperation : public CgOperation {

public:
	/**
	 * Creates a Texture2dOperation with both interior and boundary CgProgram instances.
	 * @param computationWidth the width of the computation domain.
	 * @param computationHeight the height of the computation domain.
	 * @param interior the CgProgram instance used for interior computations.
	 * @param boundary the CgProgram instance used for boundary computations.
	 * @param normalizedTexCoords whether normalized texture coordinates is used.	         
	 */
	Texture2dOperation(unsigned int computationWidth, unsigned int computationHeight,
	                   CgProgram *interior, CgProgram *boundary,
	                   bool normalizedTexCoords = false);

	/**
	 * Creates a Texture2dOperation with only an interior CgProgram instance.
	 * @param computationWidth the width of the computation domain.
	 * @param computationHeight the height of the computation domain.
	 * @param interior the CgProgram instance used for interior computations.
	 * @param interiorOnly if false a quad filling the entire computation domain is drawn,
	 *                     otherwise the boundary frame is omitted.
	 * @param normalizedTexCoords whether normalized texture coordinates is used.	         
	 */
	Texture2dOperation(unsigned int computationWidth, unsigned int computationHeight,
	                   CgProgram *interior, bool interiorOnly = false,
	                   bool normalizedTexCoords = false);

private:
	// execution parameters
	float minX, minY, maxX, maxY;
	float minS, minT, maxS, maxT;
	bool interiorOnly;

	void initExecutionParameters();

	void executeInteriorProgram();
	void executeBoundaryProgram();
};



}

#endif