#include "EventHandlerImpl.h"

#include "Events.h"

namespace SDL {
	
EventHandlerImpl::EventHandlerImpl(bool repeat) : repeat(repeat), repeatActive(false)
{
	std::fill(pressed, pressed + 256, false);
}


void EventHandlerImpl::handleEvents(EventHandler *handler, float timedelta)
{
	SDL_Event event;

	const float sleepTime = 0.25f;

	while (SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_QUIT:
			handler->quit();
			break;
		case SDL_KEYDOWN:
			activeKey = event.key.keysym.sym;
			keySymMap[activeKey] = getCharCode(event);
			dispatchKeyPress(handler, activeKey);
			pressed[keySymMap[activeKey]] = true;
			repeatActive = repeat;
			timedelta = 0;
			timeLeft = sleepTime;
			break;
		case SDL_KEYUP:
			pressed[keySymMap[event.key.keysym.sym]] = false;
			repeatActive = false;
			break;
		default:
			break;
		}
	}

	if (repeatActive)
		handleRepeat(handler, timedelta);	
}

unsigned char EventHandlerImpl::getCharCode(const SDL_Event &event) const
{
	switch (event.key.keysym.sym) {
	case SDLK_UP:
		return KEY_UP;
	case SDLK_DOWN:
		return KEY_DOWN;
	case SDLK_LEFT:
		return KEY_LEFT;
	case SDLK_RIGHT:
		return KEY_RIGHT;
	default:
		return static_cast<unsigned char>(event.key.keysym.unicode);
	}
}

void EventHandlerImpl::handleRepeat(EventHandler *handler, float timedelta)
{
	const float repeatTime = 0.05f;

	if (timeLeft < timedelta) {
		timedelta -= timeLeft;
		timeLeft = repeatTime;

		dispatchKeyPress(handler, activeKey);

		handleRepeat(handler, timedelta);
	} else
		timeLeft -= timedelta;
}


bool EventHandlerImpl::isPressed(unsigned char key) const
{
	return pressed[key];
}


void EventHandlerImpl::dispatchKeyPress(EventHandler *handler, const SDLKey &key)
{
	switch (key) {
	case SDLK_ESCAPE:
		handler->quit();
		break;
	case SDLK_RETURN:
		handler->handleEnter();
		break;
	case SDLK_BACKSPACE:
		handler->handleBackspace();
		break;
	case SDLK_TAB:
		handler->handleTab();
		break;
	case SDLK_UP:
		handler->handleUpArrow();
		break;
	case SDLK_DOWN:
		handler->handleDownArrow();
		break;
	default:
		handler->handleChar(keySymMap[key]);
		break;
	}
}

}

