#include "OpenGLContext.h"

#include "SDL.h"
#include "SDL_opengl.h"

namespace SDL {

bool OpenGLContext::taken = false;

OpenGLContext::OpenGLContext(unsigned int width, unsigned int height,
							 const std::string &caption,
							 bool fullscreen) :
  screen(0)
{
  if (!taken && SDL_Init(SDL_INIT_VIDEO) >= 0) {
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE,     8);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,   8);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,    8);
    SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE,   8);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,  16);
    SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, 32);

    SDL_EnableUNICODE(1);

	SDL_WM_SetCaption(caption.c_str(), 0);

    if (fullscreen)
      screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_FULLSCREEN);
    else
      screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL);

    taken = true;
  }
}



OpenGLContext::~OpenGLContext()
{
  if (screen) {
    taken = false;

    SDL_Quit();
  }
}



bool OpenGLContext::good() const
{
  return screen != 0;
}



bool OpenGLContext::lock() const
{
  if (SDL_MUSTLOCK(screen)) 
    if (SDL_LockSurface(screen) < 0) 
      return false;

  return true;
}



void OpenGLContext::unlock() const
{
  if (SDL_MUSTLOCK(screen)) 
    SDL_UnlockSurface(screen);
}



void OpenGLContext::flip() const
{
  SDL_GL_SwapBuffers();
}

}

