#ifndef SIMPLEXML_XMLDOCUMENT_H
#define SIMPLEXML_XMLDOCUMENT_H

#include <exception>
#include <string>

/**
 * The SimpleXml namespace.
 */
namespace SimpleXml {

class XmlNode;

class SimpleXmlLoadException : public std::exception {
public:
	SimpleXmlLoadException(const std::string &xmlFile) :
		errorStr("Failed loading xml file: " + xmlFile + ".") {}
	const char* what() const { return errorStr.c_str(); }
private:
	std::string errorStr;
};

class SimpleXmlParseException : public std::exception {
public:
	SimpleXmlParseException(const std::string &errorStr) : 
		errorStr(errorStr) {}
	const char* what() const { return errorStr.c_str(); }
private:
	std::string errorStr;
};

class XmlDocument {
public:
	XmlDocument(const std::string &xmlFile);
	
	XmlNode* getRootNode();
private:
	void loadFile();
	void parseDocument(const std::string &document);
	bool findNextTag(const std::string &document, int startIndex,
	                 std::string *tagName, int *startOfTag, int *endOfTag, bool *isStartTag);
	bool isLegalFirstLetter(char c);
	bool isLegalNameChar(char c);
	std::string readDocument(std::istream &is);

	XmlNode *rootNode;
	std::string xmlFile;
};

}

#endif