#include "XmlNode.h"

#include <string>

namespace SimpleXml {

XmlNode::XmlNode(const std::string &name) :
	parent(0),
	firstChild(0),
	lastChild(0),
	nextSibling(0),
	previousSibling(0),
	name(name)
{
	
}

XmlNode* XmlNode::getParent()
{
	return parent;
}

XmlNode* XmlNode::getFirstChild()
{
	return firstChild;
}

XmlNode* XmlNode::getFirstChild(const std::string &nodeName)
{	
	for(XmlNode *current = firstChild; current != 0; current = current->nextSibling) {
		
		if (current->getName() == nodeName)
			return current;
	}

	return 0;
}

XmlNode* XmlNode::getNextSibling()
{
	return nextSibling;
}

XmlNode* XmlNode::getNextSibling(const std::string &nodeName)
{
	for(XmlNode *current = nextSibling; current != 0; current = current->nextSibling) {
		
		if (current->getName() == nodeName)
			return current;
	}

	return 0;
}

const std::string& XmlNode::getName() const
{
	return name;
}

const std::string& XmlNode::getValue() const
{
	return value;
}

void XmlNode::setValue(const std::string &value)
{
	this->value = value;
}

XmlNode* XmlNode::appendChild(const std::string &childName)
{
	XmlNode *newChild = new XmlNode(childName);
	newChild->parent = this;
	
	if (!hasChildNodes()) {
		firstChild = newChild;
		lastChild = newChild;
	} else {
		lastChild->nextSibling = newChild;
		newChild->previousSibling = lastChild;
		lastChild = newChild;
	}

	return lastChild;
}

bool XmlNode::hasChildNodes() const
{
	return firstChild != 0;
}

/*private:
	XmlNode *parent;
	XmlNode *firstChild;
	XmlNode *lastChild;
	XmlNode *nextSibling;
	XmlNode *previousSibling;

	std::string name;
	std::string value;
};*/

}
