#ifndef BUFFER_H
#define BUFFER_H

#include <algorithm>
#include <cassert>
#include <vector>

template <typename T>
class Buffer {
public:
  Buffer(unsigned int width, unsigned int height);

  T &operator() (unsigned int x, unsigned int y) {
    return mBuffer[y * mWidth + x];
  }
  
  const T &operator() (unsigned int x, unsigned int y) const {
    return mBuffer[y * mWidth + x];
  }

  T &operator() (unsigned int i) { return mBuffer[i]; } 
  const T &operator() (unsigned int i) const { return mBuffer[i]; }

  unsigned int width() const { return mWidth; }
  unsigned int height() const { return mHeight; }
  
  unsigned int size() const { return mSize; }

	float bilerp(float x, float y) const;

  void swap(Buffer<T> &buffer);

  std::vector<T> &data() { return mBuffer; }
  const std::vector<T> &data() const { return mBuffer; }
  
private:
  unsigned int mWidth, mHeight, mSize;

  std::vector<T> mBuffer;
};



template <typename T>
Buffer<T>::Buffer(unsigned int width, unsigned int height) :
  mWidth(width),
  mHeight(height),
  mSize(width * height),
  mBuffer(std::vector<T>(mSize))
{
}



template <typename T>
float Buffer<T>::bilerp(float x, float y) const
{
	unsigned int left = static_cast<unsigned int>(x);
	unsigned int right = left + 1;

	unsigned int bottom = static_cast<unsigned int>(y);
	unsigned int top = bottom + 1;

	float u = x - left;
	float v = y - bottom;

	float tl = mBuffer[top * mWidth + left];
	float tr = mBuffer[top * mWidth + right];
	float bl = mBuffer[bottom * mWidth + left];
	float br = mBuffer[bottom * mWidth + right];

	return      v  * (u * tr + (1 - u) * tl) +
		     (1 - v) * (u * br + (1 - u) * bl);
}



template <typename T>
void Buffer<T>::swap(Buffer<T> &buffer)
{
  std::swap(mWidth, buffer.mWidth);
  std::swap(mHeight, buffer.mHeight);
  std::swap(mSize, buffer.mSize);
  std::swap(mBuffer, buffer.mBuffer);
}

#endif
