#include "Color.h"

#include <algorithm>
#include <cmath>
#include <iostream>

namespace Util {

Color::Color(float red, float green, float blue, float alpha) :
	red(red), green(green), blue(blue), alpha(alpha)
{}

Color::Color() :
	red(0), green(0), blue(0), alpha(0)
{}

float Color::getRed() const
{
	return red;
}

float Color::getGreen() const
{
	return green;
}

float Color::getBlue() const
{
	return blue;
}

float Color::getAlpha() const
{
	return alpha;
}

void Color::setRed(float red)
{
	this->red = red;
}

void Color::setGreen(float green)
{
	this->green = green;
}

void Color::setBlue(float blue)
{
	this->blue = blue;
}

void Color::setAlpha(float alpha)
{
	this->alpha = alpha;
}

float Color::diff(const Color &c) const
{
	return std::max(std::fabs(red - c.red),
	       std::max(std::fabs(green - c.green),
		   std::max(std::fabs(blue - c.blue),
	                std::fabs(alpha - c.alpha))));
}

const Color Color::operator*(float f) const
{
	return Color(red * f, green * f, blue * f, alpha * f);
}

std::ostream &operator<<(std::ostream &out, const Color& c)
{
	return out << c.getRed() << ' ' << c.getGreen() << ' ' << c.getBlue() << ' ' << c.getAlpha();
}

std::istream &operator>>(std::istream &in, Color& c)
{
	float red, green, blue, alpha;

	if (in >> red)
		c.setRed(red);

	if (in >> green)
		c.setGreen(green);

	if (in >> blue)
		c.setBlue(blue);

	if (in >> alpha)
		c.setAlpha(alpha);

	return in;
}

}


