#ifndef UTIL_COLOR_H
#define UTIL_COLOR_H

#include <iosfwd>

/**
 * The Util namespace.
 */
namespace Util {

/**
 * The class Color represents a RGBA color value.
 */
class Color {
public:
	/**
	 * Constructs a Color with the specified values.
	 * @param red the red color value.
	 * @param green the green color value.
	 * @param blue the blue color value.
	 * @param alpha the alpha value.
	 */
	Color(float red, float green, float blue, float alpha);

	/**
	 * Constructs a Color with all values set to zero.
	 */
	Color();


	/**
	 * Returns the red color value.
	 * @return the red color value.
	 */
	float getRed() const;

	/**
	 * Returns the green color value.
	 * @return the green color value.
	 */
	float getGreen() const;

	/**
	 * Returns the blue color value.
	 * @return the blue color value.
	 */
	float getBlue() const;

	/**
	 * Returns the alpha value.
	 * @return the alpha value.
	 */
	float getAlpha() const;

	/**
	 * Sets the red color value.
	 * @param red the new red color value.
	 */
	void setRed(float red);

	/**
	 * Sets the green color value.
	 * @param green the new green color value.
	 */
	void setGreen(float green);

	/**
	 * Sets the blue color value.
	 * @param blue the new blue color value.
	 */
	void setBlue(float blue);

	/**
	 * Sets the alpha value.
	 * @param alpha the new alpha value.
	 */
	void setAlpha(float alpha);

	/**
	 * Computes the four value differences (red, green, blue, alpha) between
	 * this Color and the specified Color, and returns the maximum value difference.
	 * @return the maximum value difference.
	 */
	float diff(const Color &c) const;

	const Color Color::operator*(float f) const;

private:
	float red, green, blue, alpha;
};

std::ostream &operator<<(std::ostream &out, const Color& c);
std::istream &operator>>(std::istream &in, Color& c);
}

#endif
