#ifndef RANDOM_H
#define RANDOM_H

#include <cstdlib>

class Random {
public:
  Random(int seed = 1);

  int next();

  int seed() const;
  void reseed(int seed);

  static Random &global();
  static int getMaxValue();

private:
  int mCurrent, mSeed;

  static const int mModulus, mMultiplier;
};

bool nextBool(Random &rnd);
double nextDouble(Random &rnd);
float nextFloat(Random &rnd);

/// Generates a random variable from the exponential distribution.
/// P(x) = gamma * exp(-gamma * x)
double nextExponential(Random &rnd, double gamma);

/// Generates a random variable from the gaussian distribution.
/// P(x) = [1 / sqrt(2 * pi)] * exp(-x² / 2) 
double nextGaussian(Random &rnd);

inline float randf()
{
  return std::rand() / (1.0f + RAND_MAX);
}

#endif
